@@ -10,8 +10,9 @@ from numba.special import *
 import os
 import sys
 import logging
-from numba import typesystem
+from numba import utils, typesystem
 
+PY3 = sys.version_info[0] == 3
 
 
 def get_include():
@@ -56,21 +57,36 @@ def _config_logger():
 
 _config_logger()
 
+
 from . import  special
 from numba.typesystem import *
-from . import decorators
 from numba.minivect.minitypes import FunctionType
-from .decorators import *
 from numba.error import *
 
+from numba.containers.typedlist import typedlist
+from numba.containers.typedtuple import typedtuple
+from numba.typesystem import map_dtype
+from numba.type_inference.module_type_inference import (is_registered,
+                                                        register,
+                                                        register_inferer,
+                                                        get_inferer,
+                                                        register_unbound,
+                                                        register_callable)
+from numba.typesystem.typeset import *
+
+from numba.codegen import translate
+from numba.decorators import *
+from numba import decorators
+
+__all__ = typesystem.__all__ + decorators.__all__ + special.__all__
+__all__.extend(["numeric", "floating", "complextypes"])
+
+
 # doctest compatible for jit or autojit numba functions
 from numba.tests.test_support import testmod
 
 EXCLUDE_TEST_PACKAGES = ["bytecode"]
 
-def split_path(path):
-    return path.split(os.sep)
-
 def exclude_package_dirs(dirs):
     for exclude_pkg in EXCLUDE_TEST_PACKAGES:
         if exclude_pkg in dirs:
@@ -103,7 +119,6 @@ else:
                            for signal_name, signal_code in vars(signal).items()
                                if signal_name.startswith("SIG"))
 
-
 def test(whitelist=None, blacklist=None):
     import os
     from os.path import dirname, join
@@ -125,22 +140,26 @@ def test(whitelist=None, blacklist=None):
                     continue
 
                 run += 1
-                print "running %-60s" % (modname,),
+                sys.stdout.write("running %-61s" % (modname,))
                 process = subprocess.Popen([sys.executable, '-m', modname],
                                            stdout=subprocess.PIPE,
-                                           stderr=subprocess.STDOUT)
+                                           stderr=subprocess.PIPE)
                 out, err = process.communicate()
 
                 if process.returncode == 0:
-                    print "SUCCESS"
+                    print("SUCCESS")
                 else:
-                    print "FAILED: %s" % map_returncode_to_message(
-                                                process.returncode)
-                    print out, err
-                    print "-" * 80
+                    print("FAILED: %s" % map_returncode_to_message(
+                                                process.returncode))
+                    if PY3:
+                        out = str(out, encoding='UTF-8')
+                        err = str(err, encoding='UTF-8')
+                    print(out)
+                    print(err)
+                    print("-" * 80)
                     failed += 1
 
-    print "ran test files: failed: (%d/%d)" % (failed, run)
+    print("ran test files: failed: (%d/%d)" % (failed, run))
     return failed
 
 def nose_run(module=None):
@@ -152,16 +171,3 @@ def nose_run(module=None):
 
     result = nose.main()
     return len(result.errors), len(result.failures)
-
-__all__ = typesystem.__all__ + decorators.__all__ + special.__all__
-
-from numba.typesystem import map_dtype
-from numba.type_inference.module_type_inference import (is_registered,
-                                                        register,
-                                                        register_inferer,
-                                                        get_inferer,
-                                                        register_unbound,
-                                                        register_callable)
-
-from numba.typesystem.typeset import *
-__all__.extend(["numeric", "floating", "complextypes"])
