@@ -1,4 +1,4 @@
-from external import ExternalFunction
+from .external import ExternalFunction
 from numba import *
 
 class ofunc(ExternalFunction):
@@ -67,6 +67,14 @@ class PyObject_SetItem(ExternalFunction):
     arg_types = [object_, object_, object_]
     return_type = int_
 
+class PyObject_GetIter(ExternalFunction):
+    arg_types = [object_]
+    return_type = object_
+
+class PyIter_Next(ExternalFunction):
+    arg_types = [object_]
+    return_type = object_
+
 class PySlice_New(ExternalFunction):
     arg_types = [object_, object_, object_]
     return_type = object_
@@ -138,8 +146,13 @@ class PyComplex_FromCComplex(ExternalFunction):
     arg_types = [complex128]
     return_type = object_
 
-class PyInt_FromString(ExternalFunction):
-    arg_types = [c_string_type, c_string_type.pointer(), int_]
+if not PY3:
+    class PyInt_FromString(ExternalFunction):
+        arg_types = [c_string_type, c_string_type.pointer(), int_]
+        return_type = object_
+
+class PyLong_FromString(ExternalFunction):
+    arg_types = [c_string_type, c_string_type.pointer(), long_]
     return_type = object_
 
 class PyFloat_FromString(ExternalFunction):
@@ -184,7 +197,10 @@ def create_binary_pyfunc(name):
 create_binary_pyfunc('PyNumber_Add')
 create_binary_pyfunc('PyNumber_Subtract')
 create_binary_pyfunc('PyNumber_Multiply')
-create_binary_pyfunc('PyNumber_Divide')
+if PY3:
+    create_binary_pyfunc('PyNumber_TrueDivide')
+else:
+    create_binary_pyfunc('PyNumber_Divide')
 create_binary_pyfunc('PyNumber_Remainder')
 
 class PyNumber_Power(ExternalFunction):
@@ -211,6 +227,12 @@ class PyObject_IsTrue(ExternalFunction):
     arg_types = [object_]
     return_type = int_
 
+class PyObject_RichCompareBool(ExternalFunction):
+    arg_types = [object_, object_, int_]
+    return_type = int_
+    badval = -1
+    # check_pyerr_occurred = True
+
 __all__ = [k for k, v in globals().items()
            if (v != ExternalFunction
                and isinstance(v, type)
