@@ -1,21 +1,22 @@
 import ast, inspect, os
-import textwrap
-
-from collections import defaultdict
-from numba import *
-from . import naming
-from .minivect import minitypes
 import logging
+import textwrap
+from collections import defaultdict
+
+from numba import *
+from numba.minivect import minitypes
 from numba import numbawrapper
 
+import llvm.core
+
 logger = logging.getLogger(__name__)
 
 try:
     from meta.decompiler import decompile_func
-except Exception, exn:
-    logger.warn("Could not import Meta - AST translation will not work "
-                "if the source is not available.")
-    decompile_func = None
+except Exception as exn:
+    def decompile_func(*args, **kwargs):
+        raise Exception("Could not import Meta -- Cannot recreate source "
+                        "from bytecode")
 
 def fix_ast_lineno(tree):
     # NOTE: A hack to fix assertion error in debug mode due to bad lineno.
@@ -62,16 +63,19 @@ def _get_ast(func):
     except IOError:
         return decompile_func(func)
     else:
-        source = textwrap.dedent(source)
         # Split off decorators
+        # TODO: This is not quite correct, we can have comments or strings
+        # starting at column 0 and an indented function !
+        source = textwrap.dedent(source)
         decorators = 0
-        while not source.startswith('def'): # decorator can have multiple lines
+        while not source.lstrip().startswith('def'): # decorator can have multiple lines
+            assert source
             decorator, sep, source = source.partition('\n')
             decorators += 1
         module_ast = ast.parse(source)
 
         # fix line numbering
-        lineoffset = func.func_code.co_firstlineno + decorators
+        lineoffset = func.__code__.co_firstlineno + decorators
         ast.increment_lineno(module_ast, lineoffset)
 
         assert len(module_ast.body) == 1
@@ -80,45 +84,6 @@ def _get_ast(func):
         assert isinstance(func_def, ast.FunctionDef)
         return func_def
 
-def _infer_types(context, func, restype=None, argtypes=None, **kwargs):
-    import numba.type_inference.infer as type_inference
-
-    ast = _get_ast(func)
-    func_signature = minitypes.FunctionType(return_type=restype,
-                                            args=argtypes)
-    return type_inference.run_pipeline(context, func, ast,
-                                       func_signature, **kwargs)
-
-
-def _compile(context, func, restype=None, argtypes=None, ctypes=False,
-             compile_only=False, name=None, **kwds):
-    """
-    Compile a numba annotated function.
-
-        - decompile function into a Python ast
-        - run type inference using the given input types
-        - compile the function to LLVM
-    """
-    import numba.ast_translate as translate
-
-    func_signature, symtab, ast = _infer_types(context, func,
-                                               restype, argtypes, **kwds)
-    func_name = name or naming.specialized_mangle(func.__name__, func_signature.args)
-    func_signature.name = func_name
-
-    t = translate.LLVMCodeGenerator(
-        context, func, ast, func_signature=func_signature,
-        symtab=symtab, **kwds)
-    t.translate()
-
-    if compile_only:
-        return func_signature, t.lfunc, None
-    if ctypes:
-        ctypes_func = t.get_ctypes_func(kwds.get('llvm', True))
-        return func_signature, t.lfunc, ctypes_func
-    else:
-        return func_signature, t.lfunc, t.build_wrapper_function()
-
 live_objects = [] # These are never collected
 
 def keep_alive(py_func, obj):
@@ -135,8 +100,9 @@ class FunctionCache(object):
     """
     Cache for compiler functions, declared external functions and constants.
     """
-    def __init__(self, context):
+    def __init__(self, context=None, env=None):
         self.context = context
+        self.env = env
 
         # All numba-compiled functions
         # (py_func) -> (arg_types, flags) -> (signature, llvm_func, ctypes_func)
@@ -182,45 +148,18 @@ class FunctionCache(object):
         '''
         return self.__compiled_funcs[func]
 
-    def register_specialization(self, func, compiled, argtypes, flags):
-        argtypes_flags = tuple(argtypes), flags
+    def register_specialization(self, func_env):
+        func = func_env.func
+        argtypes = func_env.func_signature.args
+        compiled = (
+            func_env.func_signature,
+            func_env.lfunc,
+            func_env.numba_wrapper_func,
+        )
+
+        # Sanity check
+        assert isinstance(func_env.func_signature, minitypes.FunctionType)
+        assert isinstance(func_env.lfunc, llvm.core.Function)
+
+        argtypes_flags = tuple(argtypes), None
         self.__compiled_funcs[func][argtypes_flags] = compiled
-
-    def compile_function(self, func, argtypes, restype=None,
-                         ctypes=False, **kwds):
-        """
-        Compile a python function given the argument types. Compile only
-        if not compiled already, and only if it is registered to the function
-        cache.
-
-        Returns a triplet of (signature, llvm_func, python_callable)
-        `python_callable` may be the original function, or a ctypes callable
-        if the function was compiled.
-        """
-        # For NumbaFunction, we get the original python function.
-        func = getattr(func, 'py_func', func)
-        assert func in self.__compiled_funcs, func
-
-        # get the compile flags
-        flags = None # stub
-
-        # Search in cache
-        result = self.get_function(func, argtypes, flags)
-        if result is not None:
-            sig, trans, pycall = result
-            return sig, trans.lfunc, pycall
-
-        # Compile the function
-        from numba import pipeline
-
-        compile_only = getattr(func, '_numba_compile_only', False)
-        kwds['compile_only'] = kwds.get('compile_only', compile_only)
-
-        assert kwds.get('llvm_module') is None, kwds.get('llvm_module')
-
-        compiled = pipeline.compile(self.context, func, restype, argtypes,
-                                    ctypes=ctypes, **kwds)
-        func_signature, translator, ctypes_func = compiled
-    
-        self.register_specialization(func, compiled, func_signature.args, flags)
-        return func_signature, translator.lfunc, ctypes_func
