@@ -4,12 +4,13 @@ Utility module containing common (to Numba) LLVM types.
 '''
 # ______________________________________________________________________
 
-import sys
 import ctypes
 import struct as struct_
-import platform
 import llvm.core as lc
 
+from numba import utils
+from numba.typedefs import _trace_refs_, PyObject_HEAD
+
 import logging
 
 logger = logging.getLogger(__name__)
@@ -38,17 +39,13 @@ _double = lc.Type.double()
 _complex64 = lc.Type.struct([_float, _float])
 _complex128 = lc.Type.struct([_double, _double])
 
-_pyobject_head = [_intp, lc.Type.pointer(_int32)]
-_pyobject_head_struct = lc.Type.struct(_pyobject_head)
-_pyobject_head_struct_p = lc.Type.pointer(_pyobject_head_struct)
 
-if hasattr(sys, 'getobjects'):
-    _trace_refs_ = True
-    _pyobject_head = [lc.Type.pointer(_int32),
-                      lc.Type.pointer(_int32)] + \
-                      _pyobject_head
-else:
-    _trace_refs_ = False
+def to_llvm(type):
+    return type.to_llvm(utils.context)
+
+_pyobject_head = [to_llvm(ty) for name, ty in PyObject_HEAD.fields]
+_pyobject_head_struct = to_llvm(PyObject_HEAD)
+_pyobject_head_struct_p = lc.Type.pointer(_pyobject_head_struct)
 
 _head_len = len(_pyobject_head)
 _numpy_struct = lc.Type.struct(_pyobject_head+\
@@ -83,9 +80,9 @@ class _LLVMCaster(object):
     def __init__(self, builder):
         self.builder = builder
 
-    def cast(self, lvalue, dst_ltype):
+    def cast(self, lvalue, dst_ltype, *args, **kws):
         src_ltype = lvalue.type
-        return self.build_cast(self.builder, lvalue, dst_ltype)
+        return self.build_cast(self.builder, lvalue, dst_ltype, *args, **kws)
 
     def build_pointer_cast(_, builder, lval1, lty2):
         return builder.bitcast(lval1, lty2)
@@ -153,6 +150,14 @@ class _LLVMCaster(object):
         lkind1 = lty1.kind
         lkind2 = lty2.kind
 
+        # This looks like the wrong place to enforce this
+        # TODO: We need to pass in the numba types instead
+        # if lc.TYPE_INTEGER in (lkind1, lkind2) and 'unsigned' not in kws:
+        #     # Be strict about having `unsigned` define when
+        #     # we have integer types
+        #     raise ValueError("Unknown signedness for integer type",
+        #                      '%s -> %s' % (lty1, lty2), args, kws)
+
         if lkind1 == lkind2:
 
             if lkind1 in cls.CAST_MAP:
