@@ -4,15 +4,13 @@ ctypes function.
 """
 
 import math
+import warnings
 
-from miniutils import ctypes
-from minitypes import *
+from .miniutils import ctypes
+from .minitypes import *
 
 try:
     from ctypes import *
-    _ctypes_func_type = type(ctypes.CFUNCTYPE(ctypes.c_int))
-    _ctypes_pointer_type = type(ctypes.POINTER(ctypes.c_int))
-    _ctypes_array_type = type(ctypes.c_int * 2)
 except ImportError:
     pass
 
@@ -34,7 +32,7 @@ def convert_to_ctypes(type):
     >>> #convert_to_ctypes(complex128)
     >>> #convert_to_ctypes(complex256)
     """
-    import minitypes
+    from . import minitypes
 
     if type.is_pointer:
         return ctypes.POINTER(convert_to_ctypes(type.base_type))
@@ -71,7 +69,7 @@ def convert_to_ctypes(type):
                       ctypes.c_uint64]
         return values[item_idx]
     elif type.is_complex:
-        import complex_support
+        from . import complex_support
         if type.itemsize == 8:
             return complex_support.Complex64
         elif type.itemsize == 16:
@@ -97,52 +95,6 @@ def convert_to_ctypes(type):
     else:
         raise NotImplementedError(type)
 
-def convert_from_ctypes(type):
-    """
-    Convert a ctypes type to a minitype
-    """
-    import ctypes
-
-    ctypes_map = {
-        c_char : char,
-        c_byte : char,
-        c_ubyte : uchar,
-        c_short : short,
-        c_ushort : ushort,
-        c_int : int_,
-        c_uint : uint,
-        c_long : c_ulong,
-        c_long : long_,
-        c_ulong : ulong,
-        c_longlong : longlong,
-        c_ulonglong : ulonglong,
-
-        c_float : float_,
-        c_double : double,
-
-        c_char_p : c_string_type,
-        c_void_p : void.pointer(),
-        None : void,
-
-        py_object : object_,
-    }
-    # On MSVC Windows, these are equal
-    if c_longdouble != c_double:
-        ctypes_map[c_longdouble] = longdouble
-
-    if type in ctypes_map:
-        return ctypes_map[type]
-    elif isinstance(type, _ctypes_ptr_type):
-        return convert_from_ctypes(type._type_).pointer()
-    elif isinstance(type, _ctypes_array_type):
-        base_type = convert_from_ctypes(type._type_)
-        return CArrayType(base_type, type._length_)
-    elif isinstance(type, _ctypes_func_type):
-        restype = convert_from_ctypes(type.restype)
-        argtypes = map(convert_from_ctypes, type.argtypes)
-        return FunctionType(return_type=restype, args=argtypes)
-    else:
-        raise NotImplementedError(type)
 
 def get_ctypes_func(func, llvm_func, llvm_execution_engine, context):
     "Get a ctypes function from an llvm function"
@@ -157,7 +109,8 @@ def get_data_pointer(numpy_array, array_type):
 
 def get_pointer(context, llvm_func):
     "Get a pointer to the LLVM function (int)"
-    return context.llvm_ee.get_pointer_to_function(llvm_func)
+    from numba.codegen.llvmcontext import LLVMContextManager
+    return LLVMContextManager().execution_engine.get_pointer_to_function(llvm_func)
 
 if __name__ == '__main__':
     import doctest
