@@ -11,11 +11,11 @@ try:
 except ImportError:
     llvm = None
 
-import codegen
-import minierror
-import minitypes
-import minivisitor
-import ctypes_conversion
+from . import codegen
+from . import minierror
+from . import minitypes
+from . import minivisitor
+from . import ctypes_conversion
 
 def handle_struct_passing(builder, alloca_func, largs, signature):
     """
@@ -99,7 +99,7 @@ class LLVMCodeGen(codegen.CodeGen):
 
         llvm_fpm = llvm.passes.FunctionPassManager.new(self.llvm_module)
         # target_data = llvm.ee.TargetData(self.context.llvm_ee)
-        llvm_fpm.add(self.context.llvm_ee.target_data.clone())
+        #llvm_fpm.add(self.context.llvm_ee.target_data.clone())
         pmb = llvm.passes.PassManagerBuilder.new()
         pmb.opt_level = 3
         pmb.vectorize = True
@@ -124,13 +124,17 @@ class LLVMCodeGen(codegen.CodeGen):
         self.add_arguments(node)
         self.visit(node.body)
 
+        # print self.lfunc
         self.lfunc.verify()
         self.optimize()
         # print self.lfunc
 
         self.code.write(self.lfunc)
+
+        from numba.codegen.llvmcontext import LLVMContextManager
         ctypes_func = ctypes_conversion.get_ctypes_func(
-                    node, self.lfunc, self.context.llvm_ee, self.context)
+                    node, self.lfunc, LLVMContextManager().execution_engine,
+                                                        self.context)
         self.code.write(ctypes_func)
 
     def add_arguments(self, function):
@@ -313,7 +317,8 @@ class LLVMCodeGen(codegen.CodeGen):
 
         op = node.operator
         if (node.type.is_int or node.type.is_float) and node.operator in self._binops:
-            llvm_method_name = self._binops[op][node.type.is_int + node.type.is_signed]
+            idx = 0 if node.type.is_float else node.type.is_int + node.type.signed
+            llvm_method_name = self._binops[op][idx]
             meth = getattr(self.builder, llvm_method_name)
             if not lhs.type == rhs.type:
                 node.print_tree(self.context)
