@@ -9,15 +9,15 @@ import copy
 import string
 import types
 
-import minitypes
-import miniutils
-import minivisitor
-import specializers
-import type_promoter
-import minicode
-import codegen
-import llvm_codegen
-import graphviz
+from . import minitypes
+from . import miniutils
+from . import minivisitor
+from . import specializers
+from . import type_promoter
+from . import minicode
+from . import codegen
+from . import llvm_codegen
+from . import graphviz
 
 try:
     import llvm.core
@@ -136,7 +136,7 @@ class Context(object):
 
             self.llvm_module = llvm.core.Module.new('default_module')
             # self.llvm_ee = llvm.ee.ExecutionEngine.new(self.llvm_module)
-            self.llvm_ee = llvm.ee.EngineBuilder.new(self.llvm_module).force_jit().opt(3).create()
+            #self.llvm_ee = llvm.ee.EngineBuilder.new(self.llvm_module).force_jit().opt(3).create()
             self.llvm_fpm = llvm.passes.FunctionPassManager.new(self.llvm_module)
             self.llvm_fpm.initialize()
             if not self.debug:
@@ -204,7 +204,7 @@ class Context(object):
         context.strides_type = self.strides_type
 
         context.debug = self.debug
-        result = context.run(ast, [specializer]).next()
+        result = next(context.run(ast, [specializer]))
         _, specialized_ast, _, (proto, impl) = result
         return impl
 
@@ -433,6 +433,15 @@ class ASTBuilder(object):
         name = name or 'function'
         return self.function(name, body, args, shapevar=shapevar)
 
+    def function_from_numpy(self, name, ast, miniargs):
+        """
+        Build a minivect function from the given ast and arguments. The
+        function takes a pointer to a numpy shape (npy_intp *).
+        """
+        type = minitypes.npy_intp.pointer()
+        shape_variable = self.variable(type, 'shape')
+        return self.function(name, ast, miniargs, shapevar=shape_variable)
+
     def funcarg(self, variable, *variables):
         """
         Create a (compound) function argument consisting of one or multiple
@@ -1027,7 +1036,7 @@ class Node(miniutils.ComparableObjectMixin):
     def __eq__(self, other):
         # Don't use isinstance here, compare on exact type to be consistent
         # with __hash__. Override where sensible
-        return (type(self) is type(other) and
+        return (isinstance(self, type(other)) and
                 self.comparison_objects == other.comparison_objects)
 
     def __hash__(self):
