@@ -41,11 +41,11 @@ import struct as struct_
 import types
 import textwrap
 
-import miniutils
-import minierror
+from . import miniutils
+from . import minierror
 
-from miniutils import *
-from miniutils import have_ctypes, ctypes
+from .miniutils import *
+from .miniutils import have_ctypes, ctypes
 
 _plat_bits = struct_.calcsize('@P') * 8
 
@@ -155,7 +155,17 @@ class TypeMapper(object):
         elif isinstance(value, bool):
             return bool_
         elif isinstance(value, (int, long)):
-            return int_
+            if abs(value) < 1:
+                bits = 0
+            else:
+                bits = math.ceil(math.log(abs(value), 2))
+
+            if bits < 32:
+                return int_
+            elif bits < 64:
+                return int64
+            else:
+                raise ValueError("Cannot represent %s as int32 or int64", value)
         elif isinstance(value, complex):
             return complex128
         elif isinstance(value, str):
@@ -171,7 +181,7 @@ class TypeMapper(object):
 
     def promote_numeric(self, type1, type2):
         "Promote two numeric types"
-        type = miniutils.max([type1, type2], key=lambda type: type.rank)
+        type = max([type1, type2], key=lambda type: type.rank)
         if type1.kind != type2.kind:
             def itemsize(type):
                 return type.itemsize // 2 if type.is_complex else type.itemsize
@@ -191,7 +201,7 @@ class TypeMapper(object):
         "Promote two array types in an expression to a new array type"
         equal_ndim = type1.ndim == type2.ndim
         return ArrayType(self.promote_types(type1.dtype, type2.dtype),
-                         ndim=miniutils.max((type1.ndim, type2.ndim)),
+                         ndim=max((type1.ndim, type2.ndim)),
                          is_c_contig=(equal_ndim and type1.is_c_contig and
                                       type2.is_c_contig),
                          is_f_contig=(equal_ndim and type1.is_f_contig and
@@ -411,7 +421,7 @@ class Type(miniutils.ComparableObjectMixin):
         if not cmps:
             return id(self) == id(other)
 
-        return (type(self) is type(other) and
+        return (isinstance(self, type(other)) and
                 cmps == other.comparison_type_list)
 
     def __ne__(self, other):
@@ -473,7 +483,7 @@ class Type(miniutils.ComparableObjectMixin):
         Convert type to ctypes. The result may be cached!
         """
         if self._ctypes_type is None:
-            import ctypes_conversion
+            from . import ctypes_conversion
             self._ctypes_type = ctypes_conversion.convert_to_ctypes(self)
             self.mutated = False
 
@@ -494,6 +504,13 @@ class Type(miniutils.ComparableObjectMixin):
     def __call__(self, *args):
         """Return a FunctionType with return_type and args set
         """
+        if len(args) == 1 and not isinstance(args[0], Type):
+            # Cast in Python space
+            # TODO: Create proxy object
+            # TODO: Fully customizable type system (do this in Numba, not
+            #       minivect)
+            return args[0]
+
         return FunctionType(self, args)
 
 class KeyHashingType(Type):
@@ -611,6 +628,7 @@ class PointerType(Type):
     def __init__(self, base_type, **kwds):
         super(PointerType, self).__init__(**kwds)
         self.base_type = base_type
+        self.itemsize = struct_.calcsize("P")
 
     def __repr__(self):
         return "%s *%s" % (self.base_type, " ".join(self.qualifiers))
@@ -829,6 +847,7 @@ class VoidType(NamedType):
 
 class ObjectType(Type):
     is_object = True
+    itemsize = VoidType().pointer().itemsize
 
     def __repr__(self):
         return "PyObject *"
@@ -871,20 +890,22 @@ class FunctionType(Type):
     def __init__(self, return_type, args, name=None, is_vararg=False, **kwds):
         super(FunctionType, self).__init__(**kwds)
         self.return_type = return_type
-        self.args = args
+        self.args = tuple(args)
         self.name = name
         self.is_vararg = is_vararg
 
     def to_llvm(self, context):
         assert self.return_type is not None
         self = self.actual_signature
+        arg_types = [arg_type.pointer() if arg_type.is_function else arg_type
+                         for arg_type in self.args]
         return lc.Type.function(self.return_type.to_llvm(context),
                                 [arg_type.to_llvm(context)
-                                    for arg_type in self.args],
+                                     for arg_type in arg_types],
                                 self.is_vararg)
 
     def __repr__(self):
-        args = map(str, self.args)
+        args = [str(arg) for arg in self.args]
         if self.is_vararg:
             args.append("...")
         if self.name:
@@ -1042,6 +1063,9 @@ class struct(Type):
     def copy(self):
         return struct(self.fields, self.name, self.readonly, self.packed)
 
+    def __eq__(self, other):
+        return other.is_struct and self.fields == other.fields
+
     def __repr__(self):
         if self.name:
             name = self.name + ' '
