@@ -11,6 +11,8 @@ https://github.com/markflorisson88/cython/blob/_array_expressions/Cython/Utility
 
 import sys
 import copy
+from functools import reduce
+
 try:
     from functools import wraps
 except ImportError:
@@ -19,11 +21,11 @@ except ImportError:
             return wrapper
         return decorator
 
-import minivisitor
-import miniutils
-import minitypes
-import minierror
-import codegen
+from . import minivisitor
+from . import miniutils
+from . import minitypes
+from . import minierror
+from . import codegen
 
 strength_reduction = True
 
@@ -125,7 +127,7 @@ class BaseSpecializer(ASTMapper):
         """
         Fuse consecutive OpenMPConditionalNodes.
         """
-        import miniast
+        from . import miniast
 
         if not node.stats:
             return node
@@ -317,7 +319,7 @@ class FinalSpecializer(BaseSpecializer):
         Run any optimizations on the AST. Currently only loop-invariant code
         motion is implemented when broadcasting information is present.
         """
-        import optimize
+        from . import optimize
 
         # TODO: support vectorized specializations
         if (self.context.optimize_broadcasting and not
@@ -496,32 +498,39 @@ class FinalSpecializer(BaseSpecializer):
                 return self.outer_pointers[variable][stop - 1]
         else:
             self.outer_pointers[variable] = []
-            start = offset
+            start = max(offset - 1, 0)
 
         outer_pointers = self.outer_pointers[variable]
         temp = argument_data_pointer
         for_loops = self.function.for_loops[start:stop]
 
-        # Loop over all outer loop levels
-        for i, for_node in zip(range(start, stop), for_loops):
-            if for_node.dim < offset:
-                continue
-
-            # Allocate a temp_data_pointer on each outer loop level
-            temp = b.temp(pointer_type)
-            dim = for_node.dim - offset
-
-            if not outer_pointers: #i == offset:
-                outer_node = self.function
-                outer_pointer = self.function.args[variable.name].data_pointer
-            else:
-                outer_node = self.function.for_loops[i - 1]
-                outer_pointer = outer_pointers[-1]
-
+        def generate_temp():
             # Generate: temp_data_pointer = outer_data_pointer
+            temp = b.temp(pointer_type)
             assmt = b.assign(temp, outer_pointer)
             outer_node.prepending.stats.append(assmt)
+            return temp
 
+        # Loop over all outer loop levels
+        for i, for_node in zip(range(start, stop), for_loops):
+            # Allocate a temp_data_pointer on each outer loop level
+            if not outer_pointers:
+                outer_pointer = self.function.args[variable.name].data_pointer
+            else:
+                outer_pointer = outer_pointers[-1]
+
+            if i == 0:
+                outer_node = self.function
+            else:
+                outer_node = self.function.for_loops[i - 1]
+
+            temp = generate_temp()
+
+            if for_node.dim < offset:
+                # No stride addition needed
+                continue
+
+            dim = for_node.dim - offset
             stride = original_stride = self.strides[variable][dim]
             assert stride is not None, ('strides', self.strides[variable],
                                         'dim', dim, 'start', start,
@@ -702,7 +711,7 @@ class FinalSpecializer(BaseSpecializer):
         """
         Generate a call to PyErr_Format() to set an exception.
         """
-        from minitypes import FunctionType, object_
+        from .minitypes import FunctionType, object_
         b = self.astbuilder
 
         args = [object_] * (2 + len(node.fmt_args))
