@@ -18,22 +18,24 @@ class ClosureNode(ExprNode):
 
     _fields = []
 
-    def __init__(self, func_def, closure_type, outer_py_func, **kwargs):
+    def __init__(self, env, func_def, closure_type, outer_py_func, **kwargs):
         super(ClosureNode, self).__init__(**kwargs)
         self.func_def = func_def
         self.type = closure_type
         self.outer_py_func = outer_py_func
         self.name = self.func_def.name
 
-        self.need_numba_func = getattr(func_def, 'need_closure_wrapper', True)
+        func_env = env.translation.get_env(func_def)
+        self.need_numba_func = not func_env or func_env.need_closure_wrapper
         self.lfunc = None
         self.wrapper_func = None
         self.wrapper_lfunc = None
         self.lfunc_pointer = None
 
-        # ast and symtab after type inference
-        self.type_inferred_ast = None
-        self.symtab = None
+        # FunctionEnvironment after type inference
+        self.func_env = None
+        # self.type_inferred_ast = None
+        # self.symtab = None
 
         from numba import pipeline
         self.locals = pipeline.get_locals(func_def, None)
@@ -44,7 +46,7 @@ class ClosureNode(ExprNode):
         self.need_closure_scope = False
 
     def make_pyfunc(self):
-        d = self.outer_py_func.func_globals
+        d = self.outer_py_func.__globals__
 #        argnames = tuple(arg.id for arg in self.func_def.args.args)
 #        dummy_func_string = """
 #def __numba_closure_func(%s):
@@ -63,7 +65,7 @@ class ClosureNode(ExprNode):
         ast_mod = ast.Module(body=[self.func_def])
         numba.functions.fix_ast_lineno(ast_mod)
         c = compile(ast_mod, '<string>', 'exec')
-        exec c in d, d
+        exec(c, d, d)
         self.func_def.name = name
 
         self.py_func = d['__numba_closure_func']
@@ -96,28 +98,45 @@ class ClosureCallNode(NativeCallNode):
     Call to closure or inner function.
     """
 
+    _fields = ['func', 'args']
+
     def __init__(self, closure_type, call_node, **kwargs):
         self.call_node = call_node
-        args, keywords = call_node.args, call_node.keywords
-        args = args + self._resolve_keywords(closure_type, args, keywords)
-        super(ClosureCallNode, self).__init__(closure_type.signature, args,
-                                              llvm_func=None, **kwargs)
+        self.func = call_node.func
         self.closure_type = closure_type
 
-    def _resolve_keywords(self, closure_type, args, keywords):
-        "Map keyword arguments to positional arguments"
-        func_def = closure_type.closure.func_def
-        argnames = [name.id for name in func_def.args.args]
+        if PY3:
+            self.argnames = [name.arg for name in self.func_def.args.args]
+        else:
+            self.argnames = [name.id for name in self.func_def.args.args]
+        self.argnames = self.argnames[self.need_closure_scope:]
+        self.expected_nargs = len(self.argnames)
 
-        expected = len(argnames) - len(args)
+        args, keywords = call_node.args, call_node.keywords
+        args = args + self._resolve_keywords(args, keywords)
+        super(ClosureCallNode, self).__init__(
+                closure_type.signature, args, llvm_func=None,
+                skip_self=self.need_closure_scope, **kwargs)
+
+    @property
+    def need_closure_scope(self):
+        return self.closure_type.closure.need_closure_scope
+
+    @property
+    def func_def(self):
+        return self.closure_type.closure.func_def
+
+    def _resolve_keywords(self, args, keywords):
+        "Map keyword arguments to positional arguments"
+        expected = self.expected_nargs - len(args)
         if len(keywords) != expected:
             raise error.NumbaError(
                     self.call_node,
-                    "Expected %d arguments, got %d" % (len(argnames),
+                    "Expected %d arguments, got %d" % (self.expected_nargs,
                                                        len(args) + len(keywords)))
 
-        argpositions = dict(zip(argnames, range(len(argnames))))
-        positional = [None] * (len(argnames) - len(args))
+        argpositions = dict(zip(self.argnames, range(self.expected_nargs)))
+        positional = [None] * (self.expected_nargs - len(args))
 
         for keyword in keywords:
             argname = keyword.arg
