@@ -1,6 +1,12 @@
 from numba.nodes import *
 import numba.nodes
 
+def get_pointer_address(value, type):
+    if type.is_known_pointer:
+        return type.address
+    else:
+        return value
+
 class ConstNode(ExprNode):
     """
     Wrap a constant.
@@ -8,6 +14,12 @@ class ConstNode(ExprNode):
 
     _attributes = ['type', 'pyval']
 
+    bool_ltype = llvm.core.Type.int(1)
+    _bool_constants = {
+        False: llvm.core.Constant.int(bool_ltype, 0),
+        True: llvm.core.Constant.int(bool_ltype, 1),
+    }
+
     def __init__(self, pyval, type=None):
         if type is None:
             type = context.typemapper.from_python(pyval)
@@ -32,14 +44,18 @@ class ConstNode(ExprNode):
         elif type.is_float:
             lvalue = llvm.core.Constant.real(ltype, constant)
         elif type.is_int:
-            lvalue = llvm.core.Constant.int(ltype, constant)
+            if type.signed:
+                lvalue = llvm.core.Constant.int_signextend(ltype, constant)
+            else:
+                lvalue = llvm.core.Constant.int(ltype, constant)
         elif type.is_complex:
             real = ConstNode(constant.real, type.base_type)
             imag = ConstNode(constant.imag, type.base_type)
             lvalue = llvm.core.Constant.struct([real.value(translator),
                                                 imag.value(translator)])
         elif type.is_pointer:
-            addr_int = translator.visit(ConstNode(self.pyval, type=Py_uintptr_t))
+            addr_int = get_pointer_address(self.pyval, type)
+            addr_int = ConstNode(addr_int, type=Py_uintptr_t).value(translator)
             lvalue = translator.builder.inttoptr(addr_int, ltype)
         elif type.is_object:
             raise NotImplementedError("Use ObjectInjectNode")
@@ -48,6 +64,8 @@ class ConstNode(ExprNode):
                                             translator.llvm_module, constant)
             type_char_p = typesystem.c_string_type.to_llvm(translator.context)
             lvalue = translator.builder.bitcast(lvalue, type_char_p)
+        elif type.is_bool:
+            return self._bool_constants[self.pyval]
         elif type.is_function:
             # lvalue = map_to_function(constant, type, self.mod)
             raise NotImplementedError
