@@ -2,7 +2,7 @@ import logging
 import os
 import sys
 import functools
-from numba import decorators
+from numba import environment
 from numba import llvm_types
 import llvm.core as lc
 logger = logging.getLogger(__name__)
@@ -37,8 +37,9 @@ def which(program):
                 return exe_file
     return None
 
-_configs = {'win' : ("link.exe", ("/dll",), '.dll'),
-    'dar': ("libtool", ("-dynamic",), '.dylib'),
+_configs = {
+    'win' : ("link.exe", ("/dll",), '.dll'),
+    'dar': ("libtool", ("-dynamic", "-undefined", "dynamic_lookup"), '.so'),
     'default': ("ld", ("-shared",), ".so")
 }
 
@@ -54,6 +55,7 @@ class Compiler(object):
         self.inputs = inputs
         self.exported_signatures = {}
         self.module_name = module_name
+        self.env = environment.NumbaEnvironment.get_environment()
 
     def _emit_wrapper_init(self, llvm_module, method_defs):
         # Figure out the Python C API module creation function, and
@@ -104,12 +106,20 @@ class Compiler(object):
 
         Resets the export environment.
         '''
-        exports_env = decorators.pipeline_env.exports
+        exports_env = self.env.exports
         self.exported_signatures = exports_env.function_signature_map
+
+        # Create new module containing everything
         ret_val = lc.Module.new(self.module_name)
+
+        # Link in all exported functions
         for submod in exports_env.function_module_map.values():
             ret_val.link_in(submod)
-        if exports_env.wrap:
+
+        # Link in cbuilder utilities
+        self.env.context.cbuilder_library.link(ret_val)
+
+        if exports_env.wrap_exports:
             method_defs = []
             wrappers = exports_env.function_wrapper_map.items()
             for name, (submod, lfunc) in wrappers:
@@ -118,20 +128,20 @@ class Compiler(object):
                 method_name = ret_val.add_global_variable(
                     method_name_init.type, '.method_name')
                 method_name.initializer = method_name_init
-                method_name.linkage = lc.LINKAGE_INTERNAL
+                method_name.linkage = lc.LINKAGE_EXTERNAL
                 method_def_const = lc.Constant.struct([
                         lc.Constant.gep(method_name, [zero, zero]),
                         lc.Constant.bitcast(lfunc, llvm_types._void_star),
                         METH_VARARGS, NULL])
                 method_defs.append(method_def_const)
             self._emit_wrapper_init(ret_val, method_defs)
-        decorators._init_exports()
+        exports_env.reset()
         return ret_val
 
     def _process_inputs(self, wrap=False, **kws):
-        decorators.pipeline_env.exports.wrap = wrap
+        self.env.exports.wrap_exports = wrap
         for ifile in self.inputs:
-            execfile(ifile)
+            exec(compile(open(ifile).read(), ifile, 'exec'))
 
     def write_llvm_bitcode(self, output, **kws):
         self._process_inputs(**kws)
