@@ -1,5 +1,8 @@
-from . import utils
 import collections
+
+from numba import utils
+from numba import odict
+
 import llvm.core
 
 class Variable(object):
@@ -18,7 +21,11 @@ class Variable(object):
     _type = None
     warn_unused = True
 
+    is_global = False
+    is_builtin = False
+
     def __init__(self, type, is_constant=False, is_local=False,
+                 is_global=False, is_builtin=False,
                  name=None, lvalue=None, constant_value=None,
                  promotable_type=True, is_arg=False):
         self.type = type
@@ -29,6 +36,10 @@ class Variable(object):
 
         self.is_constant = is_constant
         self.constant_value = constant_value
+
+        self.is_global = is_global
+        self.is_builtin = is_builtin
+
         self.lvalue = lvalue
         self.promotable_type = promotable_type
         self.deleted = False
@@ -130,9 +141,9 @@ class Variable(object):
         vars(result).update(dict(kwds, **vars(variable)))
         return result
 
-    @property
-    def is_global(self):
-        return self.type and self.type.is_global
+#    @property
+#    def is_global(self):
+#        return self.type and self.type.is_global
 
     @property
     def ltype(self):
@@ -204,10 +215,13 @@ Variable.make_shared_property('need_arg_copy')
 
 class Symtab(object):
     def __init__(self, symtab_dict=None, parent=None):
-        self.symtab = symtab_dict or {}
+        self.symtab = odict.OrderedDict(symtab_dict or {})
         self.parent = parent
         self.local_counters = {}
+
+        # { (var_name, var_type) : PromotionNode }
         self.promotions = {}
+
         if parent:
             self.counters = parent.counters
             self.local_counters.update(parent.local_counters)
