@@ -28,7 +28,8 @@ def round2(a, b):
     return round(a, b)
 
 if __name__ == '__main__':
-#    round2(10.497, 2)
-#    round_val(5)
+   # round2(10.497, 2)
+   # round_val(object())
+    round_val(3.6)
     import doctest
     doctest.testmod()
