@@ -1,15 +1,15 @@
-">>> import test_circular_type_inference" # UGH nosetests :(
 from numba.tests.cfg.test_cfg_type_infer import *
-@autojit
-def test_circular_error():
+
+
+
+
+@autojit(warnstyle='simple')
+def test_circular_error(): # doctest ignores ellipses ???
     """
-    >>> try:
-    ...     test_circular_error()
-    ... except error.NumbaError, e:
-    ...     print str(e).replace('var1', '<var>').replace('var2', '<var>')
-    Warning 16:19: local variable 'var2' might be referenced before assignment
-    Warning 18:19: local variable 'var1' might be referenced before assignment
-    Unable to infer type for assignment to '<var>', insert a cast or initialize the variable.
+    >>> test_circular_error()
+    Traceback (most recent call last):
+        ...
+    NumbaError: Unable to infer type for assignment to ..., insert a cast or initialize the variable.
     """
     for i in range(10):
         if i > 5:
@@ -17,7 +17,7 @@ def test_circular_error():
         else:
             var2 = var1
 
-@autojit
+@autojit(warnstyle='simple')
 def test_simple_circular():
     """
     >>> test_simple_circular()
@@ -30,7 +30,7 @@ def test_simple_circular():
         else:
             y = x
 
-@autojit
+@autojit(warnstyle='simple')
 def test_simple_circular2():
     """
     >>> test_simple_circular2()
@@ -107,7 +107,7 @@ def test_simple_circular_binop_promotion():
 # Test Unary/Binary Operations and Comparisons
 #------------------------------------------------------------------------
 
-@autojit(warn=False)
+@autojit_py3doc(warn=False)
 def test_circular_binop():
     """
     >>> test_circular_binop()
@@ -168,7 +168,7 @@ def test_circular_compare2():
 
     return x, y
 
-@autojit(warn=False)
+@autojit_py3doc(warn=False)
 def test_circular_compare3():
     """
     >>> test_circular_compare3()
@@ -179,7 +179,7 @@ def test_circular_compare3():
     (False, 10L)
     >>> sig, syms = infer(test_circular_compare3.py_func,
     ...                   functype(None, []), warn=False)
-    >>> map(str, types(syms, 'cond', 'x'))
+    >>> [str(x) for x in types(syms, 'cond', 'x')]
     ['bool', 'Py_ssize_t']
     """
     x = 1
@@ -202,7 +202,7 @@ def test_circular_compare3():
 # Test Indexing
 #------------------------------------------------------------------------
 
-@autojit(warn=False)
+@autojit_py3doc(warn=False)
 def test_delayed_array_indexing():
     """
     >>> test_delayed_array_indexing()
@@ -224,16 +224,11 @@ def test_delayed_array_indexing():
 @autojit(warn=False)
 def test_delayed_array_slicing():
     """
-    >>> test_delayed_array_slicing()
-    [[ 0.  1.  1.  1.  1.  1.  1.  1.  1.  1.]
-     [ 1.  1.  1.  1.  1.  1.  1.  1.  1.  1.]
-     [ 1.  1.  2.  1.  1.  1.  1.  1.  1.  1.]
-     [ 1.  1.  1.  3.  1.  1.  1.  1.  1.  1.]
-     [ 1.  1.  1.  1.  4.  1.  1.  1.  1.  1.]
-     [ 1.  1.  1.  1.  1.  5.  1.  1.  1.  1.]
-     [ 1.  1.  1.  1.  1.  1.  6.  1.  1.  1.]
-     [ 1.  1.  1.  1.  1.  1.  1.  7.  1.  1.]]
-    [ 1.  1.  1.  1.  1.  1.  1.  7.  1.  1.]
+    >>> array, row = test_delayed_array_slicing()
+    >>> array2, row2 = test_delayed_array_slicing.py_func()
+    >>> assert np.all(array == array2)
+    >>> assert np.all(row == row2)
+
     >>> sig, syms = infer(test_delayed_array_slicing.py_func,
     ...                   functype(None, []), warn=False)
     >>> types(syms, 'array', 'row')
@@ -245,23 +240,17 @@ def test_delayed_array_slicing():
         array[i, i] = row[i] * i
         array = array[:, :]
 
-    print array
-    print row
+    return array, row
 
 
 @autojit(warn=False)
 def test_delayed_array_slicing2():
     """
-    >>> test_delayed_array_slicing2()
-    [[ 0.  1.  1.  1.  1.  1.  1.  1.  1.  1.]
-     [ 1.  1.  1.  1.  1.  1.  1.  1.  1.  1.]
-     [ 1.  1.  2.  1.  1.  1.  1.  1.  1.  1.]
-     [ 1.  1.  1.  3.  1.  1.  1.  1.  1.  1.]
-     [ 1.  1.  1.  1.  4.  1.  1.  1.  1.  1.]
-     [ 1.  1.  1.  1.  1.  5.  1.  1.  1.  1.]
-     [ 1.  1.  1.  1.  1.  1.  6.  1.  1.  1.]
-     [ 1.  1.  1.  1.  1.  1.  1.  7.  1.  1.]]
-    [ 1.  1.  1.  1.  1.  1.  1.  7.  1.  1.]
+    >>> array, row = test_delayed_array_slicing2()
+    >>> array2, row2 = test_delayed_array_slicing2.py_func()
+    >>> assert np.all(array == array2)
+    >>> assert np.all(row == row2)
+
     >>> sig, syms = infer(test_delayed_array_slicing.py_func,
     ...                   functype(None, []), warn=False)
     >>> types(syms, 'array', 'row')
@@ -275,10 +264,9 @@ def test_delayed_array_slicing2():
         array[i, i] = row[i] * i
         array = array[:, :]
 
-    print array
-    print row
+    return array, row
 
-@autojit(warn=False)
+@autojit_py3doc(warn=False)
 def test_delayed_string_indexing_simple():
     """
     >>> test_delayed_string_indexing_simple()
@@ -298,7 +286,7 @@ def test_delayed_string_indexing_simple():
 
     return s[1:], x
 
-@autojit(warn=False)
+@autojit_py3doc(warn=False)
 def test_delayed_string_indexing():
     """
     >>> test_delayed_string_indexing()
@@ -324,7 +312,7 @@ def test_delayed_string_indexing():
 
     return s, x
 
-@autojit(warn=False)
+@autojit_py3doc(warn=False)
 def test_delayed_string_indexing2():
     """
     >>> test_delayed_string_indexing2()
@@ -358,7 +346,7 @@ def test_string_indexing_error():
     >>> test_string_indexing_error()
     Traceback (most recent call last):
         ...
-    TypeError: Cannot promote types (char, const char *) for variable s
+    NumbaError: Cannot promote types (char, const char *) for variable s
     """
     for i in range(4):
         if i == 0:
@@ -375,7 +363,7 @@ def test_string_indexing_error2():
     >>> chr(test_string_indexing_error2())
     Traceback (most recent call last):
         ...
-    TypeError: Cannot promote types (char, const char *) for variable s
+    NumbaError: Cannot promote types (char, const char *) for variable s
     """
     for i in range(4):
         if i == 0:
@@ -405,7 +393,7 @@ def simple_func(x):
     y = x * x + 4
     return y
 
-@autojit(warn=False)
+@autojit_py3doc(warn=False)
 def test_simple_call():
     """
     >>> test_simple_call()
@@ -478,4 +466,6 @@ def infer_simple(numba_func, *varnames):
 #print vars(func)
 #test_delayed_string_indexing_simple()
 #test_delayed_array_slicing2()
+#test_simple_circular()
+#test_circular_error()
 testmod()
