@@ -1,5 +1,5 @@
-from numba import *
-
+from numba import *; from numba.error import NumbaError
+from numba.tests.test_support import rewrite_doc
 @autojit
 def error1():
     def inner():
@@ -137,13 +137,13 @@ TypeError: an integer is required
 >>> closure3()
 12
 >>> func = closure4()
->>> print func.__name__
+>>> print(func.__name__)
 inner
 >>> field, = func.__closure__._numba_attrs._fields_
 >>> import ctypes
->>> print field[0], field[1] == ctypes.c_int
-a True
->>> print func.__closure__._numba_attrs.a
+>>> print((field[0], field[1] == ctypes.c_int))
+('a', True)
+>>> print(func.__closure__._numba_attrs.a)
 12
 >>> func()
 12
@@ -161,6 +161,10 @@ a True
 21
 """
 
+
+
+
+
 @autojit
 def closure_arg(a):
     @jit('object_(object_)')
@@ -261,12 +265,19 @@ def wrong_signature(s):
     return inner
 
 __doc__ += """
->>> wrong_signature("foo")
-Traceback (most recent call last):
-    ...
-NumbaError: 258:5: Expected 1 arguments type(s), got 0
+>>> try_(wrong_signature, "foo")
+--------------------- Numba Encountered Errors or Warnings ---------------------
+<BLANKLINE>
+    @jit('object_(object_)')
+-----^
+Error 262:5: Expected 1 arguments type(s), got 0
+<BLANKLINE>
+--------------------------------------------------------------------------------
+NumbaError: 262:5: Expected 1 arguments type(s), got 0
 """
 
+
+
 @autojit
 def wrong_restype():
     @jit('object_()')
@@ -275,10 +286,15 @@ def wrong_restype():
     return inner
 
 __doc__ += """
->>> wrong_restype()
-Traceback (most recent call last):
-    ...
-NumbaError: 272:4: Function with non-void return does not return a value
+>>> try_(wrong_restype)
+--------------------- Numba Encountered Errors or Warnings ---------------------
+<BLANKLINE>
+    @jit('object_()')
+----^
+Error 283:4: Function with non-void return does not return a value
+<BLANKLINE>
+--------------------------------------------------------------------------------
+NumbaError: 283:4: Function with non-void return does not return a value
 """
 
 #
@@ -304,10 +320,15 @@ def wrong_signature2(s):
     return inner
 
 __doc__ += """
->>> wrong_signature2("foo")
-Traceback (most recent call last):
-    ...
-NumbaError: 301:5: Expected 1 arguments type(s), got 0
+>>> try_(wrong_signature2, "foo")
+--------------------- Numba Encountered Errors or Warnings ---------------------
+<BLANKLINE>
+    @object_(object_)
+-----^
+Error 317:5: Expected 1 arguments type(s), got 0
+<BLANKLINE>
+--------------------------------------------------------------------------------
+NumbaError: 317:5: Expected 1 arguments type(s), got 0
 """
 
 @autojit
@@ -316,6 +337,7 @@ def get_closure(arg):
     def closure():
         print arg
 
+    closure()
     return closure
 
 @autojit
@@ -326,6 +348,21 @@ def test_call_closure():
 __doc__ += """
 >>> test_call_closure()
 10.0
+10.0
+"""
+
+@autojit
+def test_call_closure_from_closure():
+    closure = get_closure(10.0)
+    @void()
+    def inner():
+        closure()
+    return inner
+
+__doc__ += """
+>>> test_call_closure_from_closure()()
+10.0
+10.0
 """
 
 @autojit
@@ -368,14 +405,25 @@ def test_closure_outer_locals():
 
     inner()
 
+__doc__ = rewrite_doc(__doc__)
+
+def try_(func, *args):
+    try:
+        func(*args)
+    except NumbaError, e:
+        print "%s: %s" % (type(e).__name__, e)
+
 if __name__ == '__main__':
-#    closure1 = closure_arg(1)
-#    print closure1.__name__
-#    test_call_closure()
-#    closure4()
-#    signature_dec()()
-#    test_closure_outer_locals()
-#    test_closure_loop()
-#    test_closure_outer_locals()
+    # closure1 = closure_arg(1)
+    # print closure1.__name__
+    # closure1(10)
+    # test_call_closure()
+    # closure4()
+    # signature_dec()()
+    # test_closure_outer_locals()
+    # test_closure_loop()
+    # test_closure_outer_locals()
+    # test_call_closure_from_closure()()
+    # wrong_restype()
     import numba
     numba.testmod()
