@@ -4,6 +4,7 @@
 from numba import *
 from numba import c_string_type as cstring, int_
 
+from nose.tools import nottest
 from numba.tests import test_support
 
 # ______________________________________________________________________
@@ -13,32 +14,31 @@ def convert(input_str):
 
 # ______________________________________________________________________
 
-class CStringTests(object):
-    def test_convert(self):
-        jit_convert = self.jit(argtypes = (cstring,), restype = int_)(
+def fast_convert(input_str):
+    with nopython:
+        return int(input_str[0:5])
+
+# ______________________________________________________________________
+
+class TestCString(test_support.ASTTestCase):
+    def test_convert(self, **kws):
+        jit_convert = self.jit(argtypes = (cstring,), restype = int_, **kws)(
             convert)
         for exp in xrange(10):
             test_str = str(10 ** exp)
             self.assertEqual(jit_convert(test_str), convert(test_str))
 
-# ______________________________________________________________________    
-
-class TestBytecodeCString(test_support.ByteCodeTestCase, CStringTests):
-    @test_support.checkSkipFlag("C strings not supported in bytecode "
-                                "translator.")
-    def test_convert(self, *args, **kws):
-        return super(TestBytecodeCString, self).test_convert(*args, **kws)
-
-# ______________________________________________________________________
-
-class TestASTCString(test_support.ASTTestCase, CStringTests):
-    @test_support.checkSkipFlag("Not implemented yet.")
-    def test_convert(self, *args, **kws):
-        return super(TestASTCString, self).test_convert(*args, **kws)
+    def test_convert_nopython(self, **kws):
+        jit_convert = self.jit(argtypes = (cstring,), restype = int_, **kws)(
+            fast_convert)
+        for exp in xrange(10):
+            test_str = str(10 ** exp)
+            self.assertEqual(jit_convert(test_str), convert(test_str))
 
 # ______________________________________________________________________
 
 if __name__ == "__main__":
+#    TestCString('test_convert').debug()
     test_support.main()
 
 # ______________________________________________________________________
