@@ -2,11 +2,10 @@
 
 from numba.tests.test_support import *
 
-@autojit
+@autojit_py3doc(warn=False)
 def test_modify():
     """
     >>> test_modify()
-    Warning 23:11: local variable 'i' might be referenced before assignment
     0
     1
     2
@@ -22,11 +21,10 @@ def test_modify():
     print
     return i,n
 
-@autojit
+@autojit_py3doc(warn=False)
 def test_negindex():
     """
     >>> test_negindex()
-    Warning 41:11: local variable 'i' might be referenced before assignment
     6
     5
     4
@@ -40,11 +38,10 @@ def test_negindex():
         n = 0
     return i,n
 
-@autojit
+@autojit_py3doc(warn=False)
 def test_negindex_inferred():
     """
     >>> test_negindex_inferred()
-    Warning 58:11: local variable 'i' might be referenced before assignment
     5
     4
     3
@@ -57,11 +54,10 @@ def test_negindex_inferred():
         n = 0
     return i,n
 
-@autojit
+@autojit_py3doc(warn=False)
 def test_fix():
     """
     >>> test_fix()
-    Warning 76:11: local variable 'i' might be referenced before assignment
     0
     1
     2
@@ -75,11 +71,10 @@ def test_fix():
     print
     return i
 
-@autojit
+@autojit_py3doc(warn=False)
 def test_break():
     """
     >>> test_break()
-    Warning 98:11: local variable 'i' might be referenced before assignment
     0
     1
     2
@@ -97,7 +92,7 @@ def test_break():
     print
     return i, n
 
-@autojit
+@autojit_py3doc
 def test_else_clause1():
     """
     >>> test_else_clause1()
@@ -112,7 +107,7 @@ def test_else_clause1():
     else:
         print "else clause"
 
-@autojit
+@autojit_py3doc
 def test_else_clause2():
     """
     >>> test_else_clause2()
@@ -128,7 +123,7 @@ def test_else_clause2():
     else:
         print "else clause"
 
-@autojit
+@autojit_py3doc
 def test_else_clause3():
     """
     >>> test_else_clause3()
@@ -144,12 +139,10 @@ def test_else_clause3():
     else:
         print "else clause"
 
-@autojit
+@autojit_py3doc(warn=False)
 def test_else_clause4():
     """
     >>> test_else_clause4()
-    Warning 184:43: local variable 'k' might be referenced before assignment
-    Warning 187:36: local variable 'j' might be referenced before assignment
     inner 0
     i 0
     else clause 1 0 9
@@ -190,7 +183,7 @@ def test_else_clause4():
     else:
         print "else clause"
 
-@autojit
+@autojit_py3doc
 def test_return():
     """
     >>> test_return()
@@ -208,7 +201,7 @@ def test_return():
     print
     return "FAILED!"
 
-@autojit
+@autojit_py3doc
 def test_return2():
     """
     >>> test_return2()
