@@ -14,15 +14,15 @@ myglobal = 20
 
 @autojit(backend='ast')
 def get_list(x):
-    return [1L, "hello", 2.0, x]
+    return [1, "hello", 2.0, x]
 
 @autojit(backend='ast')
 def get_tuple(x):
-    return (1L, "hello", 2.0, x)
+    return (1, "hello", 2.0, x)
 
 @autojit(backend='ast')
 def get_dict(x):
-    return {"hello": 1L, 2.0: x}
+    return {"hello": 1, 2.0: x}
 
 if __name__ == '__main__':
     import doctest
