@@ -1,4 +1,6 @@
+import os
 import sys
+import types
 import StringIO
 import unittest
 import functools
@@ -7,11 +9,32 @@ from nose.tools import nottest
 import nose.plugins.skip
 import numba
 from numba import *
-import doctest_support
+from . import doctest_support
 
 jit_ = jit
 
-import __builtin__
+if numba.PY3:
+    import re
+    def rewrite_doc(doc):
+        doc = re.sub(r'(\d+)L', r'\1', doc)
+        doc = re.sub(r'([^\.])NumbaError', r'\1numba.error.NumbaError', doc)
+        doc = re.sub(r'([^\.])InvalidTemplateError', r'\1numba.error.InvalidTemplateError', doc)
+        doc = re.sub(r'([^\.])UnpromotableTypeError', r'\1numba.minivect.minierror.UnpromotableTypeError', doc)
+        return doc
+    def autojit_py3doc(*args, **kwargs):
+        if kwargs:
+            def _inner(fun):
+                fun.__doc__ = rewrite_doc(fun.__doc__)
+                return autojit(*args, **kwargs)(fun)
+            return _inner
+        else:
+            fun = args[0]
+            fun.__doc__ = rewrite_doc(fun.__doc__)
+            return autojit(fun)
+else:
+    def rewrite_doc(doc):
+        return doc
+    autojit_py3doc = autojit
 
 class ASTTestCase(unittest.TestCase):
     jit = staticmethod(lambda *args, **kw: jit_(*args, **dict(kw, backend='ast')))
@@ -84,7 +107,7 @@ class StdoutReplacer(object):
     def __exit__(self, *args):
         sys.stdout = self.out
 
-from bytecode.test_support import ByteCodeTestCase
+from .bytecode.test_support import ByteCodeTestCase
 
 def testmod(module=None, runit=False):
     """
@@ -92,11 +115,12 @@ def testmod(module=None, runit=False):
     populates module.__test__, when run as main, runs the doctests.
     """
     if module is None:
-        modname = sys._getframe(1).f_globals['__name__']
+        mod_globals = sys._getframe(1).f_globals
+        modname = mod_globals['__name__']
         module = __import__(modname)
+        # module = types.ModuleType(modname)
+        # vars(module).update(mod_globals)
     else:
         modname = module.__name__
 
     doctest_support.testmod(module, run_doctests=runit or modname == '__main__')
-    #if modname == '__main__':
-    #numba.nose_run(mod)
