@@ -78,24 +78,21 @@ bytecode using meta. We run the following transformations:
 
 import sys
 import ast
-import copy
-import opcode
-import types
 import ctypes
-import textwrap
-import traceback
-import __builtin__ as builtins
+from numba.specialize.mathcalls import (is_intrinsic, is_math_function,
+                                        resolve_intrinsic, resolve_libc_math,
+                                        resolve_math_call)
 
 if __debug__:
     import pprint
 
 import numba
 from numba import *
-from numba import error, closure
+from numba import error, closures
 from .minivect import minierror, minitypes, codegen
-from . import macros, utils, typesystem
-from .symtab import Variable
-from . import visitors, nodes, error, functions
+from numba import macros, utils, typesystem
+from numba.symtab import Variable
+from numba import visitors, nodes, error, functions
 from numba import stdio_util, function_util
 from numba.typesystem import is_obj, promote_closest, promote_to_native
 from numba.external import utility
@@ -110,153 +107,7 @@ from numba.external import pyapi
 
 is_win32 = sys.platform == 'win32'
 
-def filter_math_funcs(math_func_names):
-    if is_win32:
-        dll = ctypes.cdll.msvcrt
-    else:
-        dll = ctypes.CDLL(None)
-
-    result_func_names = []
-    for name in math_func_names:
-        if getattr(dll, name, None) is not None:
-            result_func_names.append(name)
-
-    return result_func_names
-
-class MathMixin(object):
-    """
-    Resolve calls to math functions.
-
-    During type inference this produces MathNode nodes, and during
-    final specialization it produces LLVMIntrinsicNode and MathCallNode
-    nodes.
-    """
-
-    # sin(double), sinf(float), sinl(long double)
-    libc_math_funcs = [
-        'sin',
-        'cos',
-        'tan',
-        'acos',
-        'asin',
-        'atan',
-        'atan2',
-        'sinh',
-        'cosh',
-        'tanh',
-        'asinh',
-        'acosh',
-        'atanh',
-        'log2',
-        'log10',
-        'fabs',
-        'pow',
-        'erfc',
-        'ceil',
-        'expm1',
-        'rint',
-        'log1p',
-        'round',
-    ]
-    libc_math_funcs = filter_math_funcs(libc_math_funcs)
-
-    def get_funcname(self, py_func):
-        if py_func is np.abs:
-            return 'fabs'
-        elif py_func is np.round:
-            return 'round'
-
-        return py_func.__name__
-
-    def _is_intrinsic(self, py_func):
-        "Whether the math function is available as an llvm intrinsic"
-        intrinsic_name = 'INTR_' + self.get_funcname(py_func).upper()
-        is_intrinsic = hasattr(llvm.core, intrinsic_name)
-        return is_intrinsic # and not is_win32
-
-    def _is_math_function(self, func_args, py_func):
-        if len(func_args) == 0 or len(func_args) > 1 or py_func is None:
-            return False
-
-        type = func_args[0].variable.type
-
-        if type.is_array:
-            type = type.dtype
-            valid_type = type.is_float or type.is_int or type.is_complex
-        else:
-            valid_type = type.is_float or type.is_int
-
-        is_intrinsic = self._is_intrinsic(py_func)
-
-        math_name = self.get_funcname(py_func)
-        is_math = math_name in self.libc_math_funcs
-        if is_math and valid_type:
-            math_name = self.math_suffix(math_name, type)
-            is_math = filter_math_funcs([math_name])
-
-        return valid_type and (is_intrinsic or is_math)
-
-    def _resolve_intrinsic(self, args, py_func, signature):
-        func_name = self.get_funcname(py_func).upper()
-        return nodes.LLVMIntrinsicNode(signature, args, func_name=func_name)
-
-    def math_suffix(self, name, type):
-        if name == 'abs':
-            name = 'fabs'
-
-        if type.itemsize == 4:
-            name += 'f' # sinf(float)
-        elif type.itemsize == 16:
-            name += 'l' # sinl(long double)
-        return name
-
-    def _resolve_libc_math(self, args, py_func, signature):
-        arg_type = signature.args[0]
-        name = self.math_suffix(self.get_funcname(py_func), arg_type)
-        return nodes.MathCallNode(signature, args, llvm_func=None,
-                                  py_func=py_func, name=name)
-
-    def _resolve_math_call(self, call_node, py_func):
-        "Resolve calls to math functions to llvm.log.f32() etc"
-        # signature is a generic signature, build a correct one
-        orig_type = type = call_node.args[0].variable.type
-
-        if type.is_int:
-            type = double
-        elif type.is_array and type.dtype.is_int:
-            type = type.copy(dtype=double)
-
-        signature = minitypes.FunctionType(return_type=type, args=[type])
-        result = nodes.MathNode(py_func, signature, call_node.args[0])
-        return result
-
-    def _binop_type(self, x, y):
-        "Binary result type for math operations"
-        x_type = x.variable.type
-        y_type = y.variable.type
-        dst_type = self.context.promote_types(x_type, y_type)
-        type = dst_type
-        if type.is_int:
-            type = double
-
-        signature = minitypes.FunctionType(return_type=type, args=[type, type])
-        return dst_type, type, signature
-
-    def pow(self, node, power, mod=None):
-        name = 'pow'
-        dst_type, pow_type, signature = self._binop_type(node, power)
-        args = [node, power]
-        if pow_type.is_float and mod is None:
-            result = self._resolve_intrinsic(args, pow, signature)
-        else:
-            if mod is not None:
-                args.append(mod)
-            result = nodes.call_pyfunc(pow, args)
-
-        return nodes.CoercionNode(result, dst_type)
-
-
-class BuiltinResolverMixinBase(MathMixin):
+class BuiltinResolverMixinBase(object):
     """
     Base class for mixins resolving calls to built-in functions.
 
@@ -290,17 +141,17 @@ class BuiltinResolverMixinBase(MathMixin):
         if result is None:
             result = nodes.call_pyfunc(func, node.args)
 
-        return result
+        return nodes.CoercionNode(result, node.type)
 
-    def _expect_n_args(self, func, node, n):
-        if not isinstance(n, tuple):
-            n = (n,)
+def resolve_pow(type, args):
+    have_mod = len(args) == 3
 
-        if len(node.args) not in n:
-            expected = " or ".join(map(str, n))
-            raise error.NumbaError(
-                node, "builtin %s expects %s arguments" % (func.__name__,
-                                                           expected))
+    if (type.is_int or type.is_float) and not have_mod and not is_win32:
+        result = resolve_intrinsic(args, pow, type)
+    else:
+        result = nodes.call_pyfunc(pow, args)
+
+    return nodes.CoercionNode(result, type)
 
 class LateBuiltinResolverMixin(BuiltinResolverMixinBase):
     """
@@ -308,13 +159,11 @@ class LateBuiltinResolverMixin(BuiltinResolverMixinBase):
     """
 
     def _resolve_abs(self, func, node, argtype):
-        self._expect_n_args(func, node, 1)
-
-        is_math = self._is_math_function(node.args, abs)
+        is_math = is_math_function(node.args, abs)
 
         # TODO: generate efficient inline code
         if is_math and argtype.is_float:
-            return self._resolve_math_call(node, abs)
+            return resolve_math_call(node, abs)
         elif is_math and argtype.is_int:
             if argtype.signed:
                 type = promote_closest(self.context, argtype, [long_, longlong])
@@ -331,180 +180,37 @@ class LateBuiltinResolverMixin(BuiltinResolverMixinBase):
         return None
 
     def _resolve_round(self, func, node, argtype):
-        self._expect_n_args(func, node, (1, 2))
-        if self._is_math_function(node.args, round):
+        if is_math_function(node.args, round):
             # round() always returns a float
-            return self._resolve_math_call(node, round)
+            return resolve_math_call(node, round)
 
         return None
 
     def _resolve_pow(self, func, node, argtype):
-        self._expect_n_args(func, node, (2, 3))
-        return self.pow(*node.args)
+        return resolve_pow(node.type, node.args)
 
+    def _resolve_int_number(self, func, node, argtype, dst_type, ext_name):
+        assert len(node.args) == 2
 
-def unpack_range_args(node):
-    start, stop, step = (nodes.const(0, Py_ssize_t),
-                         None,
-                         nodes.const(1, Py_ssize_t))
+        arg1, arg2 = node.args
+        if arg1.variable.type.is_c_string:
+            return nodes.CoercionNode(
+                nodes.ObjectTempNode(
+                    function_util.external_call(
+                        self.context,
+                        self.llvm_module,
+                        ext_name,
+                        args=[arg1, nodes.NULL, arg2])),
+                dst_type=dst_type)
 
-    if len(node.args) == 0:
-        raise error.NumbaError(node, "Expected at least one argument")
-    elif len(node.args) == 1:
-        stop, = node.args
-    elif len(node.args) == 2:
-        start, stop = node.args
-    else:
-        start, stop, step = node.args
+    def _resolve_int(self, func, node, argtype, dst_type=int_):
+        if PY3:
+            return self._resolve_int_number(func, node, argtype, long_, 'PyLong_FromString')
+        return self._resolve_int_number(func, node, argtype, int_, 'PyInt_FromString')
 
-    return [start, stop, step]
+    def _resolve_long(self, func, node, argtype, dst_type=int_):
+        return self._resolve_int_number(func, node, argtype, long_, 'PyLong_FromString')
 
-class TransformForIterable(visitors.NumbaTransformer):
-    """
-    This transforms for loops such as loops over 1D arrays:
-
-            for value in my_array:
-                ...
-
-        into
-
-            for i in my_array.shape[0]:
-                value = my_array[i]
-    """
-
-    def visit_For(self, node):
-        if node.iter.type.is_range:
-            #
-            ### Handle range iteration
-            #
-            self.generic_visit(node)
-
-            temp = nodes.TempNode(node.target.type, 'target_temp')
-            nsteps = nodes.TempNode(Py_ssize_t, 'nsteps')
-            start, stop, step = unpack_range_args(node.iter)
-
-            if isinstance(step, nodes.ConstNode):
-                have_step = step.pyval != 1
-            else:
-                have_step = True
-
-            start, stop, step = [nodes.CloneableNode(n)
-                                     for n in (start, stop, step)]
-
-            if have_step:
-                compute_nsteps = """
-                    $length = {{stop}} - {{start}}
-                    {{nsteps}} = $length / {{step}}
-                    if {{nsteps_load}} * {{step}} != $length: #$length % {{step}}:
-                        # Test for truncation
-                        {{nsteps}} = {{nsteps_load}} + 1
-                    # print "nsteps", {{nsteps_load}}
-                """
-            else:
-                compute_nsteps = "{{nsteps}} = {{stop}} - {{start}}"
-
-            if node.orelse:
-                else_clause = "else: {{else_body}}"
-            else:
-                else_clause = ""
-
-            templ = textwrap.dedent("""
-                %s
-                {{temp}} = 0
-                while {{temp_load}} < {{nsteps_load}}:
-                    {{target}} = {{start}} + {{temp_load}} * {{step}}
-                    {{body}}
-                    {{temp}} = {{temp_load}} + 1
-                %s
-            """) % (textwrap.dedent(compute_nsteps), else_clause)
-
-            # Leave the bodies empty, they are already analyzed
-            body = ast.Suite(body=[])
-            else_body = ast.Suite(body=[])
-
-            # Substitute template and type infer
-            result = self.run_template(
-                templ, vars=dict(length=Py_ssize_t),
-                start=start, stop=stop, step=step,
-                nsteps=nsteps.store(), nsteps_load=nsteps.load(),
-                temp=temp.store(), temp_load=temp.load(),
-                target=node.target,
-                body=body, else_body=else_body)
-
-            # Patch the body and else clause
-            body.body.extend(node.body)
-            else_body.body.extend(node.orelse)
-
-            # Patch cfg block of target variable to the 'body' block of the
-            # while
-            node.target.variable.block = node.if_block
-
-            # Create the place to jump to for 'continue'
-            while_node = result.body[-1]
-            assert isinstance(while_node, ast.While)
-
-            target_increment = while_node.body[-1]
-            assert isinstance(target_increment, ast.Assign)
-            #incr_block = nodes.LowLevelBasicBlockNode(node=target_increment,
-            #                                          name='for_increment')
-            node.incr_block.body = [target_increment]
-            while_node.body[-1] = node.incr_block
-            while_node.continue_block = node.incr_block
-
-            # Patch the while with the For nodes cfg blocks
-            attrs = dict(vars(node), **vars(while_node))
-            while_node = nodes.build_while(**attrs)
-            result.body[-1] = while_node
-
-            return result
-
-        elif node.iter.type.is_array and node.iter.type.ndim == 1:
-            # Convert 1D array iteration to for-range and indexing
-            logger.debug(ast.dump(node))
-
-            orig_target = node.target
-            orig_iter = node.iter
-
-            # replace node.target with a temporary
-            target_name = orig_target.id + '.idx'
-            target_temp = nodes.TempNode(Py_ssize_t)
-            node.target = target_temp.store()
-
-            # replace node.iter
-            call_func = ast.Name(id='range', ctx=ast.Load())
-            call_func.type = typesystem.RangeType()
-            call_func.variable = Variable(call_func.type)
-
-            shape_index = ast.Index(nodes.ConstNode(0, typesystem.Py_ssize_t))
-            shape_index.type = typesystem.npy_intp
-            stop = ast.Subscript(value=nodes.ShapeAttributeNode(orig_iter),
-                                 slice=shape_index,
-                                 ctx=ast.Load())
-            stop.type = typesystem.intp
-            stop.variable = Variable(stop.type)
-            call_args = [nodes.ConstNode(0, typesystem.Py_ssize_t),
-                         nodes.CoercionNode(stop, typesystem.Py_ssize_t),
-                         nodes.ConstNode(1, typesystem.Py_ssize_t),]
-
-            node.iter = ast.Call(func=call_func, args=call_args)
-            node.iter.type = call_func.type
-
-            node.index = target_temp.load(invariant=True)
-            # add assignment to new target variable at the start of the body
-            index = ast.Index(value=node.index)
-            index.type = target_temp.type
-            subscript = ast.Subscript(value=orig_iter,
-                                      slice=index, ctx=ast.Load())
-            subscript.type = orig_iter.variable.type.dtype
-            subscript.variable = Variable(subscript.type)
-            coercion = nodes.CoercionNode(subscript, orig_target.type)
-            assign = ast.Assign(targets=[orig_target], value=subscript)
-
-            node.body = [assign] + node.body
-
-            return self.visit(node)
-        else:
-            raise error.NumbaError("Unsupported for loop pattern")
 
 class ResolveCoercions(visitors.NumbaTransformer):
 
@@ -516,7 +222,7 @@ class ResolveCoercions(visitors.NumbaTransformer):
 
         node_type = node.node.type
         dst_type = node.dst_type
-        if __debug__ and logger.getEffectiveLevel() < logging.DEBUG:
+        if __debug__ and self.env and self.env.debug_coercions:
             logger.debug('coercion: %s --> %s\n%s',
                          node_type, dst_type, utils.pformat_ast(node))
 
@@ -539,8 +245,10 @@ class ResolveCoercions(visitors.NumbaTransformer):
                                        "coerced to a pointer type")
             result = self.visit(nodes.NULL.coerce(dst_type))
         elif node_type.is_numeric and dst_type.is_bool:
-            result = self.visit(ast.Compare(node.node, [ast.NotEq()],
-                                            [nodes.const(0, node_type)]))
+            to_bool = ast.Compare(node.node, [ast.NotEq()],
+                                  [nodes.const(0, node_type)])
+            to_bool = nodes.typednode(to_bool, bool_)
+            result = self.visit(to_bool)
         elif node_type.is_c_string and dst_type.is_numeric:
             # TODO: int <-> string conversions are explicit, this should not
             # TODO: be a coercion
@@ -557,7 +265,7 @@ class ResolveCoercions(visitors.NumbaTransformer):
                     cvtobj = function_util.external_call(
                                               self.context,
                                               self.llvm_module,
-                                              'PyInt_FromString',
+                                              'PyInt_FromString' if not PY3 else 'PyLong_FromString',
                                               args=[node.node, nodes.NULL,
                                                     nodes.const(10, int_)])
                 else:
@@ -572,29 +280,26 @@ class ResolveCoercions(visitors.NumbaTransformer):
             result = self.visit(node)
         else:
             self.generic_visit(node)
-#            if not node.node.type == node_type:
-#                result = self.visit(node)
+
             if dst_type == node.node.type:
                 result = node.node
             else:
                 result = node
 
-        if __debug__ and logger.getEffectiveLevel() < logging.DEBUG:
+        if __debug__ and self.env and self.env.debug_coercions:
             logger.debug('result = %s', utils.pformat_ast(result))
 
         return result
 
     def _get_int_conversion_func(self, type, funcs_dict):
-        type = self.context.promote_types(type, long_)
-        if type in funcs_dict:
-            return funcs_dict[type]
-
-        if type.itemsize == long_.itemsize:
+        type = promote_to_native(type)
+        if type.itemsize <= long_.itemsize:
             types = [ulong, long_]
         else:
+            assert type.itemsize > long_.itemsize
             types = [ulonglong, longlong]
 
-        return self._get_int_conversion_func(types[type.signed], funcs_dict)
+        return funcs_dict[types[type.signed]]
 
     def visit_CoerceToObject(self, node):
         new_node = node
@@ -659,6 +364,16 @@ class ResolveCoercions(visitors.NumbaTransformer):
                                                   args=[node])
         return result
 
+    def coerce_to_function_pointer(self, node, jit_func_type, func_pointer_type):
+        jit_func = jit_func_type.jit_func
+        if jit_func.signature != func_pointer_type.base_type:
+            raise error.NumbaError(node,
+                                   "Cannot coerce jit funcion %s to function of type %s" % (
+                                       jit_func, func_pointer_type))
+        pointer = self.env.llvm_context.get_pointer_to_function(jit_func.lfunc)
+        new_node = nodes.const(pointer, func_pointer_type)
+        return new_node
+
     def visit_CoerceToNative(self, node):
         """
         Try to perform fast coercion using e.g. PyLong_AsLong(), with a
@@ -702,8 +417,12 @@ class ResolveCoercions(visitors.NumbaTransformer):
                                                        cls.__name__,
                                                        args=[node.node])
         elif node_type.is_pointer:
-            raise error.NumbaError(node, "Obtaining pointers from objects "
-                                         "is not yet supported")
+            if from_type.is_jit_function and node_type.base_type.is_function:
+                new_node = self.coerce_to_function_pointer(
+                    node, from_type, node_type)
+            else:
+                raise error.NumbaError(node, "Obtaining pointers from objects "
+                                             "is not yet supported")
         elif node_type.is_void:
             raise error.NumbaError(node, "Cannot coerce %s to void" %
                                    (from_type,))
@@ -725,35 +444,28 @@ class ResolveCoercions(visitors.NumbaTransformer):
         new_node = self.visit(new_node)
         return new_node
 
-def badval(type):
-    if type.is_object or type.is_array:
-        value = nodes.NULL_obj
-        if type != object_:
-            value = value.coerce(type)
-    elif type.is_void:
-        value = None
-    elif type.is_float:
-        value = nodes.ConstNode(float('nan'), type=type)
-    elif type.is_int or type.is_complex:
-        # TODO: adjust for type.itemsize
-        bad = 0xbadbad # This pattern is hard to detect in llvm code
-        bad = 123456789
-        value = nodes.ConstNode(bad, type=type)
-    else:
-        value = nodes.BadValue(type)
 
-    return value
-
-class LateSpecializer(closure.ClosureCompilingMixin, ResolveCoercions,
-                      LateBuiltinResolverMixin, visitors.NoPythonContextMixin):
+class LateSpecializer(ResolveCoercions, LateBuiltinResolverMixin,
+                      visitors.NoPythonContextMixin):
 
     def visit_FunctionDef(self, node):
-        self.handle_phis()
-
         node.decorator_list = self.visitlist(node.decorator_list)
+
+        # Make sure to visit the entry block (not part of the CFG) and the
+        # first actual code block which may have synthetically
+        # inserted promotions
+        self.visit_ControlBlock(node.flow.blocks[0])
+        self.visit_ControlBlock(node.flow.blocks[1])
+
         node.body = self.visitlist(node.body)
 
         ret_type = self.func_signature.return_type
+        self.verify_context(ret_type)
+
+        self.setup_error_return(node, ret_type)
+        return node
+
+    def verify_context(self, ret_type):
         if ret_type.is_object or ret_type.is_array:
             # This will require some increfs, but allow it if people
             # use 'with python' later on. If 'with python' isn't used, a
@@ -764,74 +476,26 @@ class LateSpecializer(closure.ClosureCompilingMixin, ResolveCoercions,
             #                  "nopython context")
             pass
 
-        value = badval(ret_type)
+    def setup_error_return(self, node, ret_type):
+        """
+        Set FunctionDef.error_return to the AST statement that returns a
+        "bad value" that can be used as error indicator.
+        """
+        value = nodes.badval(ret_type)
+
         if value is not None:
             value = nodes.CoercionNode(value, dst_type=ret_type).cloneable
 
         error_return = ast.Return(value=value)
+
         if self.nopython and is_obj(self.func_signature.return_type):
             error_return = nodes.WithPythonNode(body=[error_return])
 
         node.error_return = error_return
-        return node
-
-    def handle_phi(self, node):
-        """
-        Handle phi nodes:
-
-            1) Handle incoming variables which are not initialized. Set
-               incoming_variable.uninitialized_value to a constant 'bad'
-               value (e.g. 0xbad for integers, NaN for floats, NULL for
-               objects)
-
-            2) Handle incoming variables which need promotions. An incoming
-               variable needs a promotion if it has a different type than
-               the the phi. The promotion happens in each ancestor block that
-               defines the variable which reaches us.
-
-               Promotions are set separately in the symbol table, since the
-               ancestor may not be our immediate parent, we cannot introduce
-               a rename and look up the latest version since there may be
-               multiple different promotions. So during codegen, we first
-               check whether incoming_type == phi_type, and otherwise we
-               look up the promotion in the parent block or an ancestor.
-        """
-        for parent_block, incoming_var in node.find_incoming():
-            if incoming_var.type.is_uninitialized:
-                incoming_type = incoming_var.type.base_type or node.type
-                bad = badval(incoming_type)
-                incoming_var.type.base_type = incoming_type
-                incoming_var.uninitialized_value = self.visit(bad)
-                # print incoming_var
-
-            elif not incoming_var.type == node.type:
-                # Create promotions for variables with phi nodes in successor
-                # blocks.
-                incoming_symtab = incoming_var.block.symtab
-                if (incoming_var, node.type) not in node.block.promotions:
-                    # Make sure we only coerce once for each destination type and
-                    # each variable
-                    incoming_var.block.promotions.add((incoming_var, node.type))
-
-                    # Create promotion node
-                    name_node = nodes.Name(id=incoming_var.renamed_name,
-                                           ctx=ast.Load())
-                    name_node.variable = incoming_var
-                    name_node.type = incoming_var.type
-                    coercion = self.visit(name_node.coerce(node.type))
-                    promotion = nodes.PromotionNode(node=coercion)
-
-                    # Add promotion node to block body
-                    incoming_var.block.body.append(promotion)
-                    promotion.variable.block = incoming_var.block
-
-                    # Update symtab
-                    incoming_symtab.promotions[incoming_var.name,
-                                               node.type] = promotion
-                else:
-                    promotion = incoming_symtab.lookup_promotion(
-                                     incoming_var.name, node.type)
 
+    def visit_ControlBlock(self, node):
+        # print node
+        self.visitchildren(node)
         return node
 
     def visit_While(self, node):
@@ -903,6 +567,22 @@ class LateSpecializer(closure.ClosureCompilingMixin, ResolveCoercions,
 
         return ast.Suite(body=self.visitlist(result))
 
+    def visit_Call(self, node):
+        func_type = node.func.type
+
+        if self.query(node, "is_math"):
+            assert node.func.type.is_known_value
+            result = resolve_math_call(node, node.func.type.value)
+            return self.visit(result)
+
+        elif func_type.is_builtin:
+            result = self._resolve_builtin_call_or_object(node, func_type.func)
+            result =  self.visit(result)
+            return result
+
+        self.generic_visit(node)
+        return node
+
     def visit_Tuple(self, node):
         self.check_context(node)
 
@@ -926,44 +606,13 @@ class LateSpecializer(closure.ClosureCompilingMixin, ResolveCoercions,
         self.generic_visit(node)
         return nodes.ObjectTempNode(node)
 
-    def visit_Call(self, node):
-        func_type = node.func.type
-
-        if func_type.is_builtin:
-            result = self._resolve_builtin_call_or_object(node, func_type.func)
-            result =  self.visit(result)
-            return result
-
-        self.generic_visit(node)
-        return node
-
-    def visit_NativeCallNode(self, node):
-        if is_obj(node.signature.return_type):
-            if self.nopython:
-                raise error.NumbaError(
-                    node, "Cannot call function returning object in "
-                          "nopython context")
-
-            self.generic_visit(node)
-            return nodes.ObjectTempNode(node)
-        elif node.badval is not None:
-            result = node.cloneable
-            body = nodes.CheckErrorNode(
-                        result, node.badval, node.goodval,
-                        node.exc_type, node.exc_msg, node.exc_args)
-            node = nodes.ExpressionNode(stmts=[body],
-                                        expr=result.clone)
-            return self.visit(node)
-        else:
-            self.generic_visit(node)
-            return node
-
     def visit_NativeFunctionCallNode(self, node):
         if node.signature.is_bound_method:
             assert isinstance(node.function, nodes.ExtensionMethod)
             return self.visit_ExtensionMethod(node.function, node)
 
-        return self.visit_NativeCallNode(node)
+        self.visitchildren(node)
+        return node
 
     def visit_ObjectCallNode(self, node):
         # self.generic_visit(node)
@@ -980,16 +629,21 @@ class LateSpecializer(closure.ClosureCompilingMixin, ResolveCoercions,
 
     def visit_MathNode(self, math_node):
         "Translate a nodes.MathNode to an intrinsic or libc math call"
-        if math_node.type.is_array:
+        from numba.type_inference.modules import mathmodule
+        lowerable = is_math_function([math_node.arg], math_node.py_func)
+
+        if math_node.type.is_array or not lowerable:
+            # Generate a Python call
             assert math_node.py_func is not None
             result = nodes.call_pyfunc(math_node.py_func, [math_node.arg])
-            return self.visit(result.coerce(math_node.type))
-
-        args = [math_node.arg], math_node.py_func, math_node.signature
-        if self._is_intrinsic(math_node.py_func):
-            result = self._resolve_intrinsic(*args)
+            result = result.coerce(math_node.type)
         else:
-            result = self._resolve_libc_math(*args)
+            # Lower to intrinsic or libc math call
+            args = [math_node.arg], math_node.py_func, math_node.type
+            if is_intrinsic(math_node.py_func):
+                result = resolve_intrinsic(*args)
+            else:
+                result = resolve_libc_math(*args)
 
         return self.visit(result)
 
@@ -1143,9 +797,9 @@ class LateSpecializer(closure.ClosureCompilingMixin, ResolveCoercions,
         # return nodes.ObjectTempNode(new_slice)
 
     def visit_Attribute(self, node):
-        if self.nopython:
+        if self.nopython and not node.value.type.is_module:
             raise error.NumbaError(
-                    node, "Cannot access Python attribute in nopython context")
+                    node, "Cannot access Python attribute in nopython context (%s)" % node.attr)
 
         if node.value.type.is_complex:
             value = self.visit(node.value)
@@ -1279,106 +933,12 @@ class LateSpecializer(closure.ClosureCompilingMixin, ResolveCoercions,
         result = nodes.PyArray_Empty([ndim, node.shape, dtype, is_fortran])
         return self.visit(result)
 
-    def _raise_exception(self, body, node):
-        if node.exc_type:
-            assert node.exc_msg
-
-            if node.exc_args:
-                args = [node.exc_type, node.exc_msg] + node.exc_args
-                raise_node = function_util.external_call(self.context,
-                                                         self.llvm_module,
-                                                         'PyErr_Format',
-                                                         args=args)
-            else:
-                args = [node.exc_type, node.exc_msg]
-                raise_node = function_util.external_call(self.context,
-                                                         self.llvm_module,
-                                                        'PyErr_SetString',
-                                                         args=args)
-
-            body.append(raise_node)
-
-    def _trap(self, body, node):
-        if node.exc_msg and node.print_on_trap:
-            pos = error.format_pos(node)
-            if node.exception_type:
-                exc_type = '%s: ' % node.exception_type.__name__
-            else:
-                exc_type = ''
-
-            msg = '%s%s%%s' % (exc_type, pos)
-            format = nodes.const(msg, c_string_type)
-            print_msg = function_util.external_call(self.context,
-                                                    self.llvm_module,
-                                                    'printf',
-                                                    args=[format,
-                                                          node.exc_msg])
-            body.append(print_msg)
-
-        trap = nodes.LLVMIntrinsicNode(signature=void(), args=[],
-                                       func_name='TRAP')
-        body.append(trap)
-
-    def visit_RaiseNode(self, node):
-        body = []
-        if self.nopython:
-            self._trap(body, node)
-        else:
-            self._raise_exception(body, node)
-
-        body.append(nodes.PropagateNode())
-        return ast.Suite(body=body)
-
-    def visit_CheckErrorNode(self, node):
-        if node.badval is not None:
-            badval = node.badval
-            eq = ast.Eq()
-        else:
-            assert node.goodval is not None
-            badval = node.goodval
-            eq = ast.NotEq()
-
-        test = ast.Compare(left=node.return_value, ops=[eq],
-                           comparators=[badval])
-        test.right = badval
-
-        check = nodes.build_if(test=test, body=[node.raise_node], orelse=[])
-        return self.visit(check)
-
-    def visit_PyErr_OccurredNode(self, node):
-        check_err = nodes.CheckErrorNode(
-            nodes.ptrtoint(function_util.external_call(
-                self.context,
-                self.llvm_module,
-                'PyErr_Occurred')),
-            goodval=nodes.ptrtoint(nodes.NULL))
-        result = nodes.CloneableNode(node.node)
-        result = nodes.ExpressionNode(stmts=[result, check_err],
-                                      expr=result.clone)
-        return self.visit(result)
-
     def visit_Name(self, node):
-        if node.type.is_builtin and not node.variable.is_local:
-            obj = getattr(builtins, node.name)
-            return nodes.ObjectInjectNode(obj, node.type)
+        if node.variable.is_constant:
+            obj = node.variable.constant_value
+            return nodes.const(obj, node.type)
 
-        if (is_obj(node.type) and isinstance(node.ctx, ast.Load) and
-                getattr(node, 'cf_maybe_null', False)):
-            # Check for unbound objects and raise UnboundLocalError if so
-            value = nodes.LLVMValueRefNode(Py_uintptr_t, None)
-            node.loaded_name = value
-
-            exc_msg = node.variable.name
-            if hasattr(node, 'lineno'):
-               exc_msg = '%s%s' % (error.format_pos(node), exc_msg)
-
-            check_unbound = nodes.CheckErrorNode(
-                    value, badval=nodes.const(0, Py_uintptr_t),
-                    exc_type=UnboundLocalError,
-                    exc_msg=exc_msg)
-            node.check_unbound = self.visit(check_unbound)
-
-        return super(LateSpecializer, self).visit_Name(node)
+        return node
 
     def visit_Return(self, node):
         return_type = self.func_signature.return_type
@@ -1390,18 +950,6 @@ class LateSpecializer(closure.ClosureCompilingMixin, ResolveCoercions,
         self.generic_visit(node)
         return node
 
-    def visit_Compare(self, node):
-        if node.left.type.is_object:
-            raise error.NumbaError(node,
-                                   "Comparing objects is not yet supported")
-        if node.left.type.is_pointer and node.comparators[0].type.is_pointer:
-            node.left = nodes.CoercionNode(node.left, Py_uintptr_t)
-            node.comparators = [nodes.CoercionNode(node.comparators[0],
-                                                   Py_uintptr_t)]
-
-        self.generic_visit(node)
-        return node
-
     def _object_binop(self, node, api_name):
         return self.visit(
             function_util.external_call(self.context,
@@ -1420,7 +968,10 @@ class LateSpecializer(closure.ClosureCompilingMixin, ResolveCoercions,
         return self._object_binop(node, 'PyNumber_Multiply')
 
     def _object_Div(self, node):
-        return self._object_binop(node, 'PyNumber_Divide')
+        if PY3:
+            return self._object_binop(node, 'PyNumber_TrueDivide')
+        else:
+            return self._object_binop(node, 'PyNumber_Divide')
 
     def _object_Mod(self, node):
         return self._object_binop(node, 'PyNumber_Remainder')
@@ -1454,6 +1005,9 @@ class LateSpecializer(closure.ClosureCompilingMixin, ResolveCoercions,
         return self._object_binop(node, 'PyNumber_FloorDivide')
 
     def visit_BinOp(self, node):
+        if isinstance(node.op, ast.Pow):
+            return self.visit(resolve_pow(node.type, [node.left, node.right]))
+
         self.generic_visit(node)
         if is_obj(node.left.type) or is_obj(node.right.type):
             op_name = type(node.op).__name__
@@ -1481,7 +1035,7 @@ class LateSpecializer(closure.ClosureCompilingMixin, ResolveCoercions,
                                                self.llvm_module,
                                                'PyObject_IsTrue',
                                                args=[node.operand])
-        cmpnode = nodes.Compare(callnode, [nodes.Eq()], [nodes.ConstNode(0)])
+        cmpnode = ast.Compare(callnode, [nodes.Eq()], [nodes.ConstNode(0)])
         return self.visit(nodes.IfExp(cmpnode,
                                       nodes.ObjectInjectNode(True),
                                       nodes.ObjectInjectNode(False)))
