@@ -5,12 +5,13 @@ import __builtin__
 
 import numpy as np
 
-from llvm import _ObjectCache, WeakValueDictionary
+# NOTE: llvmpy has changed.  _ObjectCache is removed
+# from llvm import _ObjectCache, WeakValueDictionary
 import llvm.core as lc
 import llvm.passes as lp
 import llvm.ee as le
 
-from utils import itercode, debugout, Complex64, Complex128
+from .utils import itercode, debugout, Complex64, Complex128
 #from ._ext import make_ufunc
 from .cfg import ControlFlowGraph
 from .llvm_types import _plat_bits, _int1, _int8, _int32, _intp, _intp_star, \
@@ -264,7 +265,7 @@ def str_to_llvmtype(str):
     elif str.startswith('arr['):
         ret_val = _numpy_array
     else:
-        raise TypeError, "Invalid Type: %s" % str
+        raise TypeError("Invalid Type: %s" % str)
     for _ in xrange(n_pointer):
         ret_val = lc.Type.pointer(ret_val)
     return ret_val
@@ -394,7 +395,7 @@ def resolve_type(arg1, arg2, builder = None):
         except TypeError:
             pass
     if typ1 is None and typ2 is None:
-        raise TypeError, "Both types not valid"
+        raise TypeError("Both types not valid")
     # The following is trying to enforce C-like upcasting rules where
     # we try to do the following:
     # * Use higher precision if the types are identical in kind.
@@ -446,7 +447,7 @@ def func_resolve_type(mod, func, args):
                 choicetype = typ
                 break
         if choicetype is None:
-            raise TypeError, "All types are unspecified"
+            raise TypeError("All types are unspecified")
         typs = [choicetype if x is None else x for x in typs]
         lfunc = map_to_function(func.val, typs, mod)
 
@@ -498,15 +499,15 @@ class _LLVMModuleUtils(object):
 
     @classmethod
     def get_string_constant(cls, module, const_str):
-        if (module, const_str) in cls.__string_constants:
-            ret_val = cls.__string_constants[(module, const_str)]
+        if (id(module), const_str) in cls.__string_constants:
+            ret_val = cls.__string_constants[(id(module), const_str)]
         else:
             lconst_str = lc.Constant.stringz(const_str)
             ret_val = module.add_global_variable(lconst_str.type, "__STR_%d" %
                                                  (len(cls.__string_constants),))
             ret_val.initializer = lconst_str
             ret_val.linkage = lc.LINKAGE_LINKONCE_ODR
-            cls.__string_constants[(module, const_str)] = ret_val
+            cls.__string_constants[(id(module), const_str)] = ret_val
         return ret_val
 
     @classmethod
@@ -916,16 +917,16 @@ class Translate(object):
         self.func_name = kws.get('name', None)
         if not self.func_name:
             self.func_name = self.func.func_name
-        self.fco = func.func_code
+        self.fco = func.__code__
         self.names = self.fco.co_names
         self.varnames = self.fco.co_varnames
         self.constants = self.fco.co_consts
-        self.costr = func.func_code.co_code
+        self.costr = func.__code__.co_code
         # Just the globals we will use
         self._myglobals = {}
         for name in self.names:
             try:
-                self._myglobals[name] = func.func_globals[name]
+                self._myglobals[name] = func.__globals__[name]
             except KeyError:
                 # Assumption here is that any name not in globals or
                 # builtins is an attribtue.
@@ -947,7 +948,7 @@ class Translate(object):
         global _ObjectCache
         setattr(_ObjectCache, '_ObjectCache__instances', WeakValueDictionary())
 
-        if kws.has_key('module'):
+        if 'module' in kws:
             self.mod = kws['module']
         else:
             self.mod = lc.Module.new('%s_mod_%x' % (func.__name__, id(self)))
@@ -1240,8 +1241,7 @@ class Translate(object):
         see if one of them has already been symbolically executed.  If
         so, return the symbolic locals recorded as leaving that basic
         block.  Returns None otherwise.'''
-        pred_list = list(preds)
-        pred_list.sort()
+        pred_list = sorted(preds)
         pred_list.reverse()
         next_locals = None
         for next_pred in pred_list:
@@ -1327,7 +1327,7 @@ class Translate(object):
                     choicetype = typ
                     break
             if choicetype is None:
-                raise TypeError, "All types are unspecified"
+                raise TypeError("All types are unspecified")
             typs = [choicetype if x is None else x for x in typs]
             lfunc = map_to_function(func.val, typs, self.mod)
 
