@@ -1,4 +1,4 @@
-import __builtin__
+import __builtin__ as builtins
 import math
 
 import numpy as np
@@ -6,9 +6,10 @@ import numpy as np
 
 import numba
 from numba import  error
+import numba.typesystem
 from numba.minivect.minitypes import *
 from numba.minivect.minitypes import map_dtype
-from numba.minivect import minitypes, minierror
+from numba.minivect import minitypes
 
 #------------------------------------------------------------------------
 # Numba's extension of the minivect type system
@@ -27,51 +28,17 @@ class NumbaKeyHashingType(minitypes.KeyHashingType):
 # Python Types
 #------------------------------------------------------------------------
 
-class ContainerListType(NumbaKeyHashingType, minitypes.ObjectType):
-
-    subtypes = ['base_type']
-
-    def __init__(self, base_type, size=-1):
-        super(ContainerListType, self).__init__()
-        self.base_type = base_type
-        self.size = size
-
-    @property
-    def key(self):
-        return self.base_type, self.size
-
-    def is_sized(self):
-        return self.size >= 0
-
-    def __repr__(self):
-        return "%s(%s, %s)" % (self.name, self.base_type, self.size)
-
-class TupleType(ContainerListType):
-
-    is_tuple = True
-    name = "tuple"
-
-class ListType(ContainerListType):
-    is_list = True
-    name = "list"
-
-class DictType(NumbaType, minitypes.ObjectType):
-    is_dict = True
-    name = "dict"
-
-    def __str__(self):
-        return "dict(%s)" % ", ".join(["..."] * self.size)
-
 class IteratorType(NumbaType, minitypes.ObjectType):
     is_iterator = True
     subtypes = ['base_type']
 
-    def __init__(self, base_type, **kwds):
+    def __init__(self, iterable_type, **kwds):
         super(IteratorType, self).__init__(**kwds)
-        self.base_type = base_type
+        self.iterable_type = iterable_type
+        self.base_type = numba.typesystem.element_type(iterable_type)
 
     def __repr__(self):
-        return "iterator<%s>" % (self.base_type,)
+        return "iterator<%s>" % (self.iterable_type,)
 
 class KnownValueType(NumbaType, minitypes.ObjectType):
     """
@@ -242,20 +209,27 @@ class NewAxisType(NumbaType, minitypes.ObjectType):
     def __repr__(self):
         return "newaxis"
 
-class GlobalType(NumbaType, minitypes.ObjectType):
+class GlobalType(KnownValueType):
     is_global = True
     name = None
 
+    def __init__(self, name, value, **kwds):
+        super(GlobalType, self).__init__(value, **kwds)
+        self.name = name
+
     def __repr__(self):
         return "global(%s)" % self.name
 
-class BuiltinType(NumbaType, minitypes.ObjectType):
+class BuiltinType(KnownValueType):
+
     is_builtin = True
 
     def __init__(self, name, **kwds):
-        super(BuiltinType, self).__init__(**kwds)
+        value = getattr(builtins, name)
+        super(BuiltinType, self).__init__(value, **kwds)
+
         self.name = name
-        self.func = getattr(__builtin__, name)
+        self.func = self.value
 
     def __repr__(self):
         return "builtin(%s)" % self.name
@@ -276,17 +250,23 @@ class NoneType(NumbaType, minitypes.ObjectType):
 # Function Types
 #------------------------------------------------------------------------
 
-class CTypesFunctionType(NumbaType, minitypes.ObjectType):
-    is_ctypes_function = True
+class PointerFunctionType(NumbaType, minitypes.ObjectType):
+    """
+    Pointer to a function at a known address represented by some Python
+    object (e.g. a ctypes or CFFI function).
+    """
 
-    def __init__(self, ctypes_func, restype, argtypes, **kwds):
-        super(CTypesFunctionType, self).__init__(**kwds)
-        self.ctypes_func = ctypes_func
-        self.signature = minitypes.FunctionType(return_type=restype,
-                                                args=argtypes)
+    is_pointer_to_function = True
+
+    def __init__(self, obj, pointer, signature, **kwds):
+        super(PointerFunctionType, self).__init__(**kwds)
+        self.obj = obj          # for debugability
+        self.pointer = pointer  # function address as an integer
+        self.signature = signature
 
     def __repr__(self):
-        return "<ctypes function %s>" % (self.signature,)
+        return "<natively callable function %s at %s>" % (self.signature,
+                                                          hex(self.pointer))
 
 class AutojitType(NumbaType, minitypes.ObjectType):
     """
@@ -331,10 +311,12 @@ class NULLType(NumbaType):
     def __repr__(self):
         return "<type(NULL)>"
 
-class CTypesPointerType(NumbaType):
-    def __init__(self, pointer_type, address, **kwds):
-        super(CTypesPointerType, self).__init__(**kwds)
-        self.pointer_type = pointer_type
+class KnownPointerType(minitypes.PointerType):
+
+    is_known_pointer = True
+
+    def __init__(self, base_type, address, **kwds):
+        super(KnownPointerType, self).__init__(base_type, **kwds)
         self.address = address
 
 class SizedPointerType(NumbaType, minitypes.PointerType):
@@ -422,7 +404,6 @@ class ReferenceType(NumbaType):
 # END OF TYPE DEFINITIONS
 #------------------------------------------------------------------------
 
-tuple_ = TupleType(object_, size=0)
 none = NoneType()
 null_type = NULLType()
 intp = minitypes.npy_intp
