@@ -37,6 +37,7 @@ haven't been processed yet, or circular dependences.
 """
 
 from numba import oset
+from numba.minivect import minierror
 from numba.typesystem import *
 
 class UninitializedType(NumbaType):
@@ -131,7 +132,7 @@ class PromotionType(UnresolvedType):
     def __init__(self, variable, context, types, assignment=False, **kwds):
         super(PromotionType, self).__init__(variable, **kwds)
         self.context = context
-        self.types = types
+        self.types = oset.OrderedSet(types)
         self.assignment = assignment
         variable.type = self
 
@@ -173,7 +174,7 @@ class PromotionType(UnresolvedType):
                     types.add(type)
 
     def find_types(self, seen):
-        types = set([self])
+        types = oset.OrderedSet([self])
         seen.add(self)
         seen.add(self.variable.deferred_type)
         self.dfs(types, seen)
@@ -181,7 +182,7 @@ class PromotionType(UnresolvedType):
         return types
 
     def find_simple(self, seen):
-        types = set()
+        types = oset.OrderedSet()
         for type in self.types:
             if type.is_promotion:
                 types.add(type.types)
@@ -240,13 +241,13 @@ class PromotionType(UnresolvedType):
                 return True
             else:
                 old_types = self.types
-                self.types = set([result_type] + unresolved_types)
+                self.types = oset.OrderedSet([result_type] + unresolved_types)
                 return old_types != self.types
 
     def simplify(self, seen=None):
         try:
             return self._simplify(seen)
-        except minierror.UnpromotableTypeError, e:
+        except minierror.UnpromotableTypeError as e:
             if self.variable.name:
                 name = "variable %s" % self.variable.name
             else:
@@ -254,7 +255,7 @@ class PromotionType(UnresolvedType):
 
             types = sorted(e.args[0], key=str)
             types = tuple(types)
-            raise TypeError("Cannot promote types %s for %s" % (types, name))
+            raise error.NumbaError("Cannot promote types %s for %s" % (types, name))
 
     @classmethod
     def promote(cls, *types):
@@ -547,14 +548,16 @@ class StronglyConnectedCircularType(UnresolvedType):
         super(StronglyConnectedCircularType, self).__init__(None, **kwds)
         self.scc = scc
 
-        types = set(scc)
+        types = oset.OrderedSet(scc)
         for type in scc:
             self.add_children(type.children - types)
             self.add_parents(type.parents - types)
 
         self.types = scc
-        self.promotions = set(type for type in scc if type.is_promotion)
-        self.reanalyzeable = set(type for type in scc if type.is_reanalyze_circular)
+        self.promotions = oset.OrderedSet(
+                type for type in scc if type.is_promotion)
+        self.reanalyzeable = oset.OrderedSet(
+                type for type in scc if type.is_reanalyze_circular)
 
     def retry_infer_reanalyzable(self):
         for reanalyzeable in self.reanalyzeable:
@@ -621,7 +624,8 @@ class StronglyConnectedCircularType(UnresolvedType):
         elif self.promotions:
             self.resolve_promotion_cycles()
         else:
-            assert False
+            # All dependencies are resolved, we are done
+            pass
 
         self.is_resolved = True
 
