@@ -95,17 +95,17 @@ class TemplateIndexType(TemplateType):
         return attrib[self.index]
 
     def __repr__(self):
-        return "%r[%d]" % (self.template_type, self.index)
+        return "%r[%r]" % (self.template_type, self.index)
 
     def __str__(self):
-        return "%s[%d]" % (self.template_type, self.index)
+        return "%s[%r]" % (self.template_type, self.index)
 
 
 
 template = TemplateType
 
 def validate_template(concrete_type, template_type):
-    if type(template_type) != type(concrete_type):
+    if not isinstance(template_type, type(concrete_type)):
         raise error.InvalidTemplateError(
                 "Type argument does not match template type: %s and %s" % (
                 concrete_type, template_type))
