@@ -1,12 +1,16 @@
+from itertools import imap, izip
 import copy
 import types
+import ctypes
+
 import llvm.core
 import numpy as np
+
 import numba
-from numba import llvm_types
-from numba.minivect.ctypes_conversion import convert_from_ctypes
-import numba.minivect.minitypes
+from numba.support.ctypes_support import is_ctypes, from_ctypes_value
+from numba.support import cffi_support
 from numba.minivect.minitypes import map_dtype, object_
+import numba.minivect.minitypes
 
 from numba import numbawrapper
 from numba.typesystem import *
@@ -15,6 +19,28 @@ import numba.typesystem
 def is_dtype_constructor(value):
     return isinstance(value, type) and issubclass(value, np.generic)
 
+def infer_container_type(typemapper, value):
+    assert isinstance(value, (tuple, list, dict))
+
+    if isinstance(value, tuple):
+        container_type = TupleType
+    elif isinstance(value, list):
+        container_type = ListType
+    else:
+        key_type = infer_container_type(typemapper, value.keys())
+        value_type = infer_container_type(typemapper, value.values())
+        return DictType(key_type, value_type, size=len(value))
+
+    if len(value) < 30:
+        # Figure out base type if the container is not too large
+        unify = typemapper.promote_types
+        base_type = reduce(unify, imap(typemapper.from_python, value))
+    else:
+        base_type = object_
+
+    return container_type(base_type, size=len(value))
+
+
 class NumbaTypeMapper(minitypes.TypeMapper):
     """
     Map types from Python values, handle type promotions, etc
@@ -27,6 +53,7 @@ class NumbaTypeMapper(minitypes.TypeMapper):
         # self.ctypes_func_type2 = type(libc.printf)
 
     def to_llvm(self, type):
+        from numba import llvm_types
         if type.is_array:
             return llvm_types._numpy_array
         elif type.is_complex:
@@ -51,19 +78,22 @@ class NumbaTypeMapper(minitypes.TypeMapper):
             return numba.typesystem.from_numpy_dtype(value)
         elif is_dtype_constructor(value):
             return numba.typesystem.from_numpy_dtype(np.dtype(value))
-        elif isinstance(value, tuple):
-            return tuple_
+        elif isinstance(value, (tuple, list, dict)):
+            return infer_container_type(self, value)
         elif isinstance(value, types.ModuleType):
             return ModuleType(value)
         # elif isinstance(value, (self.ctypes_func_type, self.ctypes_func_type2)):
-        elif hasattr(value, 'errcheck'):
-            # ugh, ctypes
-            if value.argtypes is None:
-                return object_
-
-            restype = convert_from_ctypes(value.restype)
-            argtypes = map(convert_from_ctypes, value.argtypes)
-            return CTypesFunctionType(value, restype, argtypes)
+        elif is_ctypes(value):
+            result = from_ctypes_value(value)
+            if result.is_function:
+                pointer = ctypes.cast(value, ctypes.c_void_p).value
+                return PointerFunctionType(value, pointer, result)
+            else:
+                return result
+        elif cffi_support.is_cffi_func(value):
+            signature = cffi_support.get_signature(value)
+            pointer = cffi_support.get_pointer(value)
+            return PointerFunctionType(value, pointer, signature)
         elif isinstance(value, minitypes.Type):
             return CastType(dst_type=value)
         elif hasattr(type(value), '__numba_ext_type'):
