@@ -1,9 +1,13 @@
 import ast
-from pprint import pprint, pformat
 import ast as ast_module
-import __builtin__ as builtins
+try:
+    import __builtin__ as builtins
+except ImportError:
+    import builtins
+
 from numba import functions
 from numba import nodes
+from numba.nodes.metadata import annotate, query
 from numba.typesystem.typemapper import have_properties
 
 try:
@@ -12,7 +16,7 @@ except ImportError:
     # pre-2.6
     numbers = None
 
-from numba import error
+from numba import error, PY3
 
 import logging
 logger = logging.getLogger(__name__)
@@ -36,6 +40,7 @@ class NumbaVisitorMixin(CooperativeBase):
             context, func, ast, func_signature=func_signature,
             nopython=nopython, symtab=symtab, **kwargs)
 
+        self.env = kwargs.get('env', None)
         self.context = context
         self.ast = ast
         self.function_cache = context.function_cache
@@ -57,12 +62,22 @@ class NumbaVisitorMixin(CooperativeBase):
             self.flow_block = None
 
         self.func = func
-        if func is None or not getattr(func, "__numba_valid_code_object", True):
+        if not self.valid_locals(func):
             assert isinstance(ast, ast_module.FunctionDef)
-            locals, cellvars, freevars = determine_variable_status(context, ast,
+            locals, cellvars, freevars = determine_variable_status(self.env, ast,
                                                                    self.locals)
             self.names = self.global_names = freevars
-            self.argnames = tuple(arg.id for arg in ast.args.args)
+
+            #TODO: Using a guard for PY3 does not work.
+            #Sometimes ast.args.args has objects of type ast.Name
+            tmp_ = []
+            for name in ast.args.args:
+                if hasattr(name, 'arg'):
+                    tmp_.append(name.arg)
+                else:
+                    tmp_.append(name.id)
+            self.argnames = tuple(tmp_)
+
             argnames = set(self.argnames)
             local_names = [local_name for local_name in locals
                                           if local_name not in argnames]
@@ -71,7 +86,7 @@ class NumbaVisitorMixin(CooperativeBase):
             self.cellvars = cellvars
             self.freevars = freevars
         else:
-            f_code = self.func.func_code
+            f_code = self.func.__code__
             self.names = self.global_names = f_code.co_names
             self.varnames = self.local_names = list(f_code.co_varnames)
 
@@ -86,10 +101,10 @@ class NumbaVisitorMixin(CooperativeBase):
             self.freevars = set(f_code.co_freevars)
 
         if func is None:
-            self.func_globals = kwargs.get('func_globals', {})
+            self.func_globals = kwargs.get('func_globals', None) or {}
             self.module_name = self.func_globals.get("__name__", "")
         else:
-            self.func_globals = func.func_globals
+            self.func_globals = func.__globals__
             self.module_name = self.func.__module__
 
         # Add variables declared in locals=dict(...)
@@ -100,9 +115,9 @@ class NumbaVisitorMixin(CooperativeBase):
         if self.is_closure_signature(func_signature) and func is not None:
             # If a closure is backed up by an actual Python function, the
             # closure scope argument is absent
-            from numba import closure
-            self.argnames = (closure.CLOSURE_SCOPE_ARG_NAME,) + self.argnames
-            self.varnames.append(closure.CLOSURE_SCOPE_ARG_NAME)
+            from numba import closures
+            self.argnames = (closures.CLOSURE_SCOPE_ARG_NAME,) + self.argnames
+            self.varnames.append(closures.CLOSURE_SCOPE_ARG_NAME)
 
         # Just the globals we will use
         self._myglobals = {}
@@ -139,32 +154,65 @@ class NumbaVisitorMixin(CooperativeBase):
             qname = "%s.%s" % (self.module_name, self.func_name)
         return qname
 
+    @property
+    def current_env(self):
+        return self.env.translation.crnt
+
+    def annotate(self, node, key, value):
+        annotate(self.env, node, key, value)
+
+    def query(self, node, key):
+        return query(self.env, node, key)
+
+    def error(self, node, msg):
+        "Issue a terminating error"
+        raise error.NumbaError(node, msg)
+
+    def deferred_error(self, node, msg):
+        "Issue a deferred-terminating error"
+        self.current_env.error_env.collection.error(node, msg)
+
+    def warn(self, node, msg):
+        "Issue a warning"
+        self.current_env.error_env.collection.warning(node, msg)
+
     def visit_func_children(self, node):
         self.func_level += 1
         self.generic_visit(node)
         self.func_level -= 1
         return node
 
+    def valid_locals(self, func):
+        if self.ast is None or self.env is None:
+            return True
+
+        return (func is not None and
+                query(self.env, self.ast, "__numba_valid_code_object",
+                      default=True))
+
     def invalidate_locals(self, ast=None):
         ast = ast or self.ast
-        if hasattr(ast, "variable_status_tuple"):
-            del ast.variable_status_tuple
+        if query(self.env, ast, "variable_status_tuple"):
+            # Delete variable status of the function (local/free/cell status)
+            annotate(self.env, ast, variable_status_tuple=None)
+
         if self.func and ast is self.ast:
-            setattr(self.func, "__numba_valid_code_object", False)
+            # Invalidate validity of code object
+            annotate(self.env, ast, __numba_valid_code_object=False)
 
     def _visit_overload(self, node):
         assert self._overloads
 
         try:
             return super(NumbaVisitorMixin, self).visit(node)
-        except error.NumbaError, e:
+        except error.NumbaError as e:
             # Try one of the overloads
             cls_name = type(node).__name__
             for i, cls_name in enumerate(self._overloads):
                 for overload_name, func in self._overloads[cls_name]:
                     try:
                         return func(self, node)
-                    except error.NumbaError, e:
+                    except error.NumbaError as e:
                         if i == len(self._overloads) - 1:
                             raise
 
@@ -185,20 +233,25 @@ class NumbaVisitorMixin(CooperativeBase):
     def run_template(self, s, vars=None, **substitutions):
         from numba import templating
 
-        templ = templating.TemplateContext(self.context, s)
+        func = self.func
+        if func is None:
+            d = dict(self.func_globals)
+            exec('def __numba_func(): pass', d, d)
+            func = d['__numba_func']
+
+        templ = templating.TemplateContext(self.context, s, env=self.env)
+
         if vars:
             for name, type in vars.iteritems():
                 templ.temp_var(name, type)
 
         symtab, tree = templ.template_type_infer(
                 substitutions, symtab=self.symtab,
-                closure_scope=getattr(self.ast, "closure_scope", None))
+                closure_scope=getattr(self.ast, "closure_scope", None),
+                func=func)
         self.symtab.update(templ.get_vars_symtab())
         return tree
 
-    def error(self, node, msg):
-        raise error.NumbaError(node, msg)
-
     def keep_alive(self, obj):
         """
         Keep an object alive for the lifetime of the translated unit.
@@ -249,10 +302,6 @@ class NumbaVisitorMixin(CooperativeBase):
     def visit_CloneNode(self, node):
         return node
 
-    #@property
-    #def current_scope(self):
-    #    return self.local_scopes[-1]
-
     def visit_ControlBlock(self, node):
         #self.local_scopes.append(node.symtab)
         self.setblock(node)
@@ -286,22 +335,6 @@ class NumbaVisitorMixin(CooperativeBase):
             for phi_node in block.phi_nodes:
                 self.handle_phi(phi_node)
 
-    @property
-    def type(self):
-        assert self.is_expr and len(node.body) == 1
-        return node.body[0].type
-
-    @property
-    def variable(self):
-        assert self.is_expr and len(node.body) == 1
-        return node.body[0].variable
-
-x = 0
-for i in range(10):
-    if x > 1:
-        print x
-    y = 14.2
-    x = x * y
 
 class NumbaVisitor(ast.NodeVisitor, NumbaVisitorMixin):
     "Non-mutating visitor"
@@ -381,7 +414,7 @@ class VariableFindingVisitor(NumbaVisitor):
         self.generic_visit(node)
         return node
 
-def determine_variable_status(context, ast, locals_dict):
+def determine_variable_status(env, ast, locals_dict):
     """
     Determine what category referenced and assignment variables fall in:
 
@@ -389,17 +422,19 @@ def determine_variable_status(context, ast, locals_dict):
         - free variables
         - cell variables
     """
-    from numba import pipeline
-
-    if hasattr(ast, 'variable_status_tuple'):
-        return ast.variable_status_tuple
+    variable_status = query(env, ast, 'variable_status_tuple')
+    if variable_status:
+        return variable_status
 
     v = VariableFindingVisitor()
     v.visit(ast)
 
     locals = set(v.assigned)
     locals.update(locals_dict)
-    locals.update(arg.id for arg in ast.args.args)
+    if PY3:
+        locals.update(arg.arg for arg in ast.args.args)
+    else:
+        locals.update(arg.id for arg in ast.args.args)
     locals.update(func_def.name for func_def in v.func_defs)
 
     freevars = set(v.referenced) - locals
@@ -407,10 +442,11 @@ def determine_variable_status(context, ast, locals_dict):
 
     # Compute cell variables
     for func_def in v.func_defs:
-        inner_locals_dict = pipeline.get_locals(func_def, None)
+        func_env = env.translation.make_partial_env(func_def, locals={})
+        inner_locals_dict = func_env.locals
 
         inner_locals, inner_cellvars, inner_freevars = \
-                            determine_variable_status(context, func_def,
+                            determine_variable_status(env, func_def,
                                                       inner_locals_dict)
         cellvars.update(locals.intersection(inner_freevars))
 
@@ -421,5 +457,5 @@ def determine_variable_status(context, ast, locals_dict):
 #    print ast.name, "locals", pformat(locals)
 
     # Cache state
-    ast.variable_status_tuple = locals, cellvars, freevars
+    annotate(env, ast, variable_status_tuple=(locals, cellvars, freevars))
     return locals, cellvars, freevars
